@@ -32,13 +32,20 @@ use warnings;
 
 # Preloads
 use LedgerSMB;
-use LedgerSMB::Form;
-use LedgerSMB::Sysconfig;
-use LedgerSMB::Template::UI;
-use LedgerSMB::Locale;
+use LedgerSMB::AA;
+use LedgerSMB::GL;
+use LedgerSMB::IIAA;
+use LedgerSMB::IR;
+use LedgerSMB::IS;
+use LedgerSMB::PE;
 use LedgerSMB::File;
-use LedgerSMB::Scripts::login;
+use LedgerSMB::Form;
+use LedgerSMB::Legacy_Util;
 use LedgerSMB::PGObject;
+use LedgerSMB::Scripts::login;
+use LedgerSMB::Setting;
+use LedgerSMB::Setting::Sequence;
+use LedgerSMB::Tax;
 
 
 =head1 LICENSE AND COPYRIGHT
