@@ -18,10 +18,9 @@ This module is the UI controller for the customer, vendor, etc functions; it
 use strict;
 use warnings;
 
-use Try::Tiny;
+use Feature::Compat::Try;
 
 use LedgerSMB;
-use LedgerSMB::DBObject::Pricelist;
 use LedgerSMB::Entity::Company;
 use LedgerSMB::Entity::Person;
 use LedgerSMB::Entity::Credit_Account;
@@ -34,10 +33,10 @@ use LedgerSMB::Entity::Bank;
 use LedgerSMB::Entity::Note;
 use LedgerSMB::Entity::User;
 use LedgerSMB::File;
+use LedgerSMB::I18N;
 use LedgerSMB::Magic qw( EC_EMPLOYEE );
 use LedgerSMB::Part;
 use LedgerSMB::Setting;
-use LedgerSMB::Template::UI;
 
 use LedgerSMB::old_code qw(dispatch);
 
@@ -171,6 +170,10 @@ sub _main_screen {
     $request->{target_div} ||= 'person_div' if defined $person;
     $request->{target_div} ||= 'company_div';
 
+    my @all_managers =
+        map { $_->{label} = "$_->{first_name} $_->{last_name}"; $_ }
+    ($request->call_procedure( funcname => 'employee__all_managers' ),);
+
     my @all_years =  $request->call_procedure(
               funcname => 'date_get_all_years'
     );
@@ -271,6 +274,9 @@ sub _main_screen {
         $ref->{text} = "$ref->{description}";
     }
 
+    my @taxform_list =
+        $request->call_procedure(funcname => 'tax_form__list_all');
+
     my @location_class_list =
        grep { $_->{class} =~ m/^(?:Billing|Sales|Shipping)$/ }
             $request->call_procedure(funcname => 'location_list_class');
@@ -278,12 +284,15 @@ sub _main_screen {
     my @business_types =
                $request->call_procedure(funcname => 'business_type__list');
 
+    my @sic_list = $request->call_procedure(funcname => 'sic__list');
+
     my @all_currencies =
         map { { curr => $_ } }
         (LedgerSMB::Setting->new(%$request))->get_currencies;
 
     my $default_country = $request->setting->get('default_country');
     my ($default_language) = $request->setting->get('default_language');
+    my ($earn_id) = $request->setting->get('earn_id');
 
     my $attach_level_options = [
         {text => $locale->text('Entity'), value => 1} ];
@@ -296,24 +305,24 @@ sub _main_screen {
     $request->close_form() if eval {$request->can('close_form')};
     $request->open_form() if eval {$request->can('close_form')};
 
-    opendir(my $dh2, 'UI/Contact/plugins') || die "can't opendir plugins directory: $!";
-    my @plugins = grep { /^[^.]/ && -f "UI/Contact/plugins/$_" } readdir($dh2);
+    my $ui_root = $request->{_wire}->get('ui')->{root} // './UI/';
+    opendir(my $dh2, "${ui_root}Contact/plugins") || die "can't opendir plugins directory: $!";
+    my @plugins = grep { /^[^.]/ && -f "${ui_root}Contact/plugins/$_" } readdir($dh2);
     closedir $dh2;
 
-    my @country_list = $request->call_procedure(
-                     funcname => 'location_list_country'
-      );
+    my @country_list = $request->enabled_countries->@*;
     my @entity_classes =
         map { $_->{class} = $locale->maketext($_->{class}) ; $_ }
         $request->call_procedure(
             funcname => 'entity__list_classes'
         );
+    my @eca_classes = grep { $_->{id} < 3 } @entity_classes;
 
     my $roles;
     $roles = $user->list_roles if $user;
 
     # Template info and rendering
-    my $template = LedgerSMB::Template::UI->new_UI;
+    my $template = $request->{_wire}->get('ui');
     return $template->render($request, 'Contact/contact', {
                      DIVS => \@DIVS,
                 DIV_LABEL => \%DIV_LABEL,
@@ -330,6 +339,7 @@ sub _main_screen {
               credit_list => \@credit_list,
               pricegroups => \@pricegroups,
            entity_classes => \@entity_classes,
+              eca_classes => \@eca_classes,
                 locations => \@locations,
                  contacts => \@contacts,
              bank_account => \@bank_account,
@@ -344,18 +354,22 @@ sub _main_screen {
         discount_acc_list => \@discount_acc_list,
        language_code_list => \@language_code_list,
        language_code_list => \@language_code_list,
+             taxform_list => \@taxform_list,
            all_currencies => \@all_currencies,
      attach_level_options => $attach_level_options,
                 entity_id => $entity_id,
              entity_class => $entity_class,
+                 sic_list => \@sic_list,
       location_class_list => \@location_class_list,
        contact_class_list => \@contact_class_list,
            business_types => \@business_types,
                 all_taxes => \@all_taxes,
                 all_years => \@all_years,
                all_months =>  $request->all_months->{dropdown},
+             all_managers => \@all_managers,
           default_country => $default_country,
-         default_language => $default_language
+         default_language => $default_language,
+                  earn_id => $earn_id
     });
 }
 
@@ -464,13 +478,13 @@ sub dispatch_legacy {
 
     };
 
-    my $entry = $dispatch->{$request->{action}};
+    my $entry = $dispatch->{$request->{__action}};
     return dispatch($entry->{script},
                     'add',
                     $request->{_user},
                     { %{$entry->{data}},
                       script => $entry->{script},
-                      action => 'add',
+                      __action => 'add',
                       dbh => $request->{dbh} });
 }
 
@@ -526,7 +540,7 @@ This method creates a blank screen for entering a company's information.
 
 sub add {
     my ($request) = @_;
-    $request->{target_div} = 'company_div';
+    $request->{target_div} //= 'company_div';
     return _main_screen($request, $request);
 }
 
@@ -669,6 +683,8 @@ This is a synonym of get() which is preferred to use for editing operations.
 =cut
 
 sub edit {
+    my ($request) = @_;
+    $request->{action} = 'edit';
     return get (@_);
 }
 
@@ -804,7 +820,7 @@ sub get_pricelist {
     my $pricelist = $credit->get_pricematrix;
     $request->merge($credit) if $credit;
     $request->merge($pricelist) if $pricelist;
-    my $template = LedgerSMB::Template::UI->new_UI;
+    my $template = $request->{_wire}->get('ui');
     return $template->render($request, 'Contact/pricelist', $request);
 }
 
@@ -822,42 +838,53 @@ and the description is a full text search.
 sub save_pricelist {
     my ($request) = @_;
     my $count = $request->{rowcount_pricematrix};
-    my $pricelist = LedgerSMB::DBObject::Pricelist->new(%$request);
     my @lines;
 
     # Search and populate
     if (defined $request->{"int_partnumber_tfoot_$count"}
-         or defined $request->{"description_tfoot_$count"})
-    {
-        my ($partnumber) = split(/--/, $request->{"int_partnumber_tfoot_$count"});
+         or defined $request->{"description_tfoot_$count"}) {
+        my ($partnumber) =
+            split(/--/, $request->{"int_partnumber_tfoot_$count"});
         my $part = LedgerSMB::Part->get_by_partnumber($partnumber);
-        push @lines, {
-                   parts_id => $part->{id},
-                  validfrom => $request->{"validfrom_tfoot_$count"},
-                    validto => $request->{"validto_tfoot_$count"},
-                      price => $request->{"lastcost_tfoot_$count"} ||
-                               $request->{"sellprice_tfoot_$count"},
-                   leadtime => $request->{"leadtime_tfoot_$count"},
-                        qty => $request->{"qty_tfoot_$count"},
-         } if defined $part->{id};
+        if (defined $part->{id}) {
+            push @lines, {
+                parts_id  => $part->{id},
+                validfrom => $request->{"validfrom_tfoot_$count"},
+                validto   => $request->{"validto_tfoot_$count"},
+                price     => ($request->{"lastcost_tfoot_$count"} ||
+                              $request->{"sellprice_tfoot_$count"}),
+                leadtime  => $request->{"leadtime_tfoot_$count"},
+                qty       => $request->{"qty_tfoot_$count"},
+                curr      => $request->{curr},
+            };
+        }
     }
 
     # Save rows
     for (1 .. ($count - 1)){
         my $id = $request->{"row_$_"};
         push @lines, {
-                entry_id => $id,
-                parts_id => $request->{"parts_id_$id"},
-               validfrom => $request->{"validfrom_$id"},
-                 validto => $request->{"validto_$id"},
-                   price => $request->{"lastcost_$id"} ||
-                            $request->{"sellprice_$id"},
-                leadtime => $request->{"leadtime_$id"},
-                     qty => $request->{"qty_$id"},
+            parts_id  => $request->{"parts_$id"},
+            validfrom => $request->{"validfrom_$id"},
+            validto   => $request->{"validto_$id"},
+            price     => ($request->{"lastcost_$id"} ||
+                          $request->{"sellprice_$id"}),
+            leadtime  => $request->{"leadtime_$id"},
+            qty       => $request->{"qty_$id"},
+            curr      => $request->{curr},
         };
     }
 
-    $pricelist->save(\@lines);
+    for my $line (@lines) {
+        $request->call_procedure(
+            funcname => 'pricelist__save',
+            args     => [
+                $line->{parts_id},
+                $request->{credit_id},
+                $line->@{qw(pricebreak price lead_time partnumber validfrom
+                            validto curr entry_id qty)}
+            ]);
+    }
 
     return get_pricelist($request);
 }
@@ -869,9 +896,9 @@ sub save_pricelist {
 
 sub delete_pricelist {
     my ($request) = @_;
-    my $pricelist = LedgerSMB::DBObject::Pricelist->new(%$request);
-
-    $pricelist->delete;
+    $request->call_procedure(
+        funcname => 'pricelist__delete',
+        args     => [ $request->{entry_id}, $request->{credit_id} ]);
 
     # Return to UI
     return get_pricelist($request);
@@ -886,19 +913,21 @@ This turns the employee into a user.
 sub create_user {
     my ($request) = @_;
     $request->{target_div} = 'user_div';
-    delete $request->{pls_import};
     if ($request->close_form){
        $request->{password} = $request->{reset_password};
        my $user = LedgerSMB::Entity::User->new(%$request);
        my $return_with_import;
+
+       delete $request->{pls_import}; ## remove after user-object instantiation
        try {
            $user->create($request->{reset_password});
-       } catch {
-           my $err = $_;
+       }
+       catch ($err) {
            die $err unless $err =~ /duplicate user/i;
            $request->{dbh}->rollback;
            $return_with_import = 1;
-       };
+       }
+
        if ($return_with_import){
            $request->{pls_import} = 1;
        }
@@ -906,6 +935,28 @@ sub create_user {
     return get($request);
 }
 
+=item delete_user
+
+This removes the user from the company.
+
+=cut
+
+sub delete_user {
+    my ($request) = @_;
+    $request->{target_div} = 'user_div';
+    if ($request->close_form){
+       my $user = LedgerSMB::Entity::User->new(%$request);
+       delete $request->{pls_import}; ## remove after user-object instantiation
+       try {
+           $user->delete;
+       }
+       catch ($err) {
+           $request->{dbh}->rollback;
+       }
+    }
+    return get($request);
+}
+
 =item reset_password
 
 This resets the user's password
