@@ -19,17 +19,20 @@ This module doesn't specify any methods.
 use strict;
 use warnings;
 
+use JSON::PP;
+use Feature::Compat::Try;
 
 use LedgerSMB::AA;
 use LedgerSMB::Batch;
-use LedgerSMB::DBObject::Account;
+use LedgerSMB::Company::Configuration;
 use LedgerSMB::Form;
 use LedgerSMB::GL;
+use LedgerSMB::Inventory::Adjust;
+use LedgerSMB::Inventory::Adjust_Line;
 use LedgerSMB::IR;
 use LedgerSMB::IS;
 use LedgerSMB::Magic qw( EC_VENDOR EC_CUSTOMER );
 use LedgerSMB::Setting::Sequence;
-use LedgerSMB::Template::UI;
 use LedgerSMB::Timecard;
 
 use List::MoreUtils qw{ any };
@@ -41,10 +44,10 @@ our $cols = {
     gl_multi        =>  ['debit_accno', 'credit_accno', 'amount', 'curr',
                          'amount_fx', 'reference', 'transdate', 'description',
                          'source_debit', 'source_credit', 'memo'],
-    ap_multi        =>  ['vendor', 'amount', 'account', 'ap', 'description',
-                        'invnumber', 'transdate'],
-    ar_multi        =>  ['customer', 'amount', 'account', 'ar', 'description',
-                        'invnumber', 'transdate'],
+    ap_multi        =>  ['vendor', 'amount', 'curr', 'fx_rate', 'account', 'ap',
+                        'description', 'invnumber', 'transdate'],
+    ar_multi        =>  ['customer', 'amount', 'curr', 'fx_rate', 'account', 'ar',
+                        'description', 'invnumber', 'transdate'],
     timecard        =>  ['employee', 'business_unit_id', 'transdate',
                         'partnumber', 'description', 'qty', 'non_billable',
                         'sellprice', 'allocated', 'notes', 'jctype', 'curr'],
@@ -69,8 +72,6 @@ my %template_file = (
 );
 
 
-our $ap_eca_for_inventory = '00000'; # Built in inventory adjustment accounts
-our $ar_eca_for_inventory = '00000';
 our $preprocess = {};
 our $postprocess = {};
 
@@ -132,56 +133,79 @@ sub _aa_multi {
     my $vcst = $request->{dbh}->prepare(
         'select count(*) from entity_credit_account where meta_number = ?'
         );
+    my $currst = $request->{dbh}->prepare(
+        'select count(*) from currency where curr = ?'
+    );
     for my $ref (@$entries){
+        my %entry;
+        # Reference with name instead of index for better readability
+        @entry{qw( vc amount curr fx_rate account arap
+                    description invnumber transdate )} = @$ref;
         my $pass;
-        next if $ref->[1] !~ /\d/;
-        my ($acct) = split /--/, $ref->[2];
+        next if $entry{amount} !~ /\d/;
+        my ($acct) = split /--/, $entry{account};
         $acst->execute($acct);
         ($pass) = $acst->fetchrow_array;
         $request->error("Account $acct not found") if !$pass;
-        ($acct) = split /--/, $ref->[3];  ## no critic (ProhibitMagicNumbers) sniff
+        ($acct) = split /--/, $entry{arap};  ## no critic (ProhibitMagicNumbers) sniff
         $acst->execute($acct);
         ($pass) = $acst->fetchrow_array;
         $request->error("Account $acct not found") if !$pass;
-        $vcst->execute(uc($ref->[0]));
+        $vcst->execute($entry{vc});
         ($pass) = $vcst->fetchrow_array;
         if (! $pass) {
             if ($arap eq 'ar') {
-                $request->error("Customer $ref->[0] not found");
+                $request->error("Customer $entry{vc} not found");
             } else {
-                $request->error("Vendor $ref->[0] not found");
+                $request->error("Vendor $entry{vc} not found");
             }
         }
+        # Check currency is defined in the system
+        $currst->execute($entry{curr});
+        ($pass) = $currst->fetchrow_array;
+        $request->error("Currency $entry{curr} not found") if !$pass;
     }
     for my $ref (@$entries){
+        my %entry;
+        @entry{qw( vc amount curr fx_rate account arap
+                    description invnumber transdate )} = @$ref;
+
         my $form = Form->new(); ## no critic
         $form->{dbh} = $request->{dbh};
         my $default_currency = $request->setting->get('curr');
         $form->{rowcount} = 1;
         $form->{ARAP} = uc($arap);
         $form->{batch_id} = $batch->{id};
-        $form->{customernumber} = $form->{vendornumber} = shift @$ref;
-        $form->{amount_1} = shift @$ref;
+        $form->{customernumber} = $form->{vendornumber} = $entry{vc};
+        $form->{amount_1} = $entry{amount};
         next if $form->{amount_1} !~ /\d/;
         $form->{amount_1} = $form->parse_amount(
             $request->{_user}, $form->{amount_1});
-        $form->{"$form->{ARAP}_amount_1"} = shift @$ref;
+        $form->{"$form->{ARAP}_amount_1"} = $entry{account};
         $form->{vc} = ($arap eq 'ar') ? 'customer' : 'vendor';
         $form->{arap} = $arap;
-        $form->{uc($arap)} = shift @$ref;
-        $form->{description_1} = shift @$ref;
-        $form->{invnumber} = shift @$ref;
-        $form->{transdate} = shift @$ref;
-        $form->{currency} = $default_currency;
+        $form->{uc($arap)} = $entry{arap};
+        $form->{description_1} = $entry{description};
+        $form->{invnumber} = $entry{invnumber};
+        $form->{transdate} = $entry{transdate};
+        $form->{currency} = $entry{curr};
+        $form->{exchangerate} = $entry{fx_rate};
         $form->{approved} = '0';
         $form->{defaultcurrency} = $default_currency;
+        # need to fetch due date by terms set on the ECA
+        # otherwise transaction posted with empty due date
         my $sth = $form->{dbh}->prepare(
-            'SELECT id FROM entity_credit_account
+            'SELECT id,
+              (?::date + (concat(
+                  (CASE WHEN terms IS NULL THEN 1 ELSE terms END),
+                  \' days\'))::interval) as duedate
+              FROM entity_credit_account
               WHERE entity_class = ? and meta_number = ?'
             );
-        $sth->execute( ($arap eq 'ar') ? 2 : 1,
-                       uc($form->{vendornumber}));
-        ($form->{vendor_id}) = $sth->fetchrow_array;
+        $sth->execute( $form->{transdate},
+            ($arap eq 'ar') ? 2 : 1,
+            $form->{vendornumber} );
+        ($form->{vendor_id}, $form->{duedate}) = $sth->fetchrow_array;
         $form->{customer_id} = $form->{vendor_id};
 
         # The 'AA' package is used as 'LedgerSMB::AA'
@@ -192,94 +216,32 @@ sub _aa_multi {
 }
 
 sub _inventory_single_date {
-    my ($request, $entries, $report_id, $transdate) = @_;
-    my $ar_form = Form->new(); ## no critic
-    my $ap_form = Form->new(); ## no critic
-    my $dbh = $request->{dbh};
+    my ($request, $entries, $transdate) = @_;
+    # NOTE ! This implementation largely mirrors the one
+    #  in LedgerSMB::Scripts::inventory::_lines_from_form()
 
-    $ar_form->{dbh} = $ap_form->{dbh} = $dbh;
-
-    # Needs to come *after* form initialization
-    my ($curr) = split /:/, $ap_form->get_setting('curr');
-
-
-    $ar_form->{rowcount} = $ap_form->{rowcount} = 0;
-    $ar_form->{transdate} = $ap_form->{transdate} = $transdate;
-    $ar_form->{defaultcurrency} = $ar_form->{currency} = $curr;
-    $ap_form->{defaultcurrency} = $ap_form->{currency} = $curr;
-    $ar_form->{type} = $ap_form->{type} = 'invoice';
-    # Intentionally not setting CRDATE here
-
-    my $p_info_sth = $dbh->prepare(
-        'SELECT * FROM parts WHERE partnumber = ?'
-        ) or $ap_form->dberror();
-    my $ins_sth = $dbh->prepare(
-        'INSERT INTO inventory_report_line
-                (parts_id, counted, expected, adjust_id)
-             VALUES (?, ?, ?, ?)'
-        ) or $ap_form->dberror();
-
-    my $adjustment = ($request->{stock_type} ne 'relative') ?
-        sub { my ($target, $part_info) = @_;
-              return ($target - $part_info->{onhand}); }
-        : sub { my ($target) = @_;
-                return $target; };
-
-    for my $line (@$entries){
-        next if $line->{onhand} !~ /\d/;
-
-        $p_info_sth->execute($line->{partnumber});
-        my $part = $p_info_sth->fetchrow_hashref('NAME_lc');
-        die "Part $line->{partnumber} not found"
-            unless $part;
-        my $adjust = &$adjustment( $line->{onhand}, $part);
-        my $adjust_form = ($adjust > 0) ? $ap_form : $ar_form;
-
-        my $rc = ++$adjust_form->{rowcount};
-        $adjust_form->{"id_$rc"} = $part->{id};
-        $adjust_form->{"sellprice_$rc"} = $line->{purchase_price};
-        $adjust_form->{"discount_$rc"} = 0;
-        $adjust_form->{"qty_$rc"} = abs($adjust);
-
-        $ins_sth->execute($part->{id}, $line->{onhand},
-                          $part->{onhand}, $report_id)
-            or $ap_form->dberror();
+    my $adjustment = LedgerSMB::Inventory::Adjust->new(
+        transdate => $transdate,
+        source    => 'CSV upload',
+        dbh       => $request->{dbh},
+        );
 
+    my @rows;
+    for my $entry ($entries->@*) {
+        my $part =
+            $adjustment->get_part_at_date($transdate, $entry->{partnumber});
+        my $counted  = $entry->{onhand};
+        my $expected = $part->{onhand};
+        push @rows, LedgerSMB::Inventory::Adjust_Line->new(
+            parts_id    => $part->{id},
+            partnumber  => $entry->{partnumber},
+            counted     => $counted,
+            expected    => $expected,
+            variance    => $expected - $counted
+            );
     }
-    $ar_form->{ARAP} = 'AR';
-    $ar_form->{AR} = $request->{AR};
-    $ap_form->{ARAP} = 'AP';
-    $ap_form->{AP} = $request->{AP};
-
-    # ECA
-    $ar_form->{'customernumber'} = $ar_eca_for_inventory;
-    $ap_form->{'vendornumber'} = $ap_eca_for_inventory;
-    $ar_form->get_name(undef, 'customer', 'today', 2);
-    $ap_form->get_name(undef, 'vendor', 'today', 1);
-    my $ar_eca = shift @{$ar_form->{name_list}};
-    my $ap_eca = shift @{$ap_form->{name_list}};
-    $ar_form->{customer_id} = $ar_eca->{id};
-    $ap_form->{vendor_id} = $ap_eca->{id};
-
-    # POST
-    IS->post_invoice(undef, $ar_form) ## no critic
-        if $ar_form->{rowcount};
-    IR->post_invoice(undef, $ap_form) ## no critic
-        if $ap_form->{rowcount};
-
-    $ar_form->{id} = 'NULL'
-        if ! $ar_form->{id};
-    $ap_form->{id} = 'NULL'
-        if ! $ap_form->{id};
-
-    # Now, update the report record.
-    return ($dbh->do( # These two params come from posting above, and from
-              # the db.
-              "UPDATE inventory_report
-                       SET ar_trans_id = $ar_form->{id},
-                           ap_trans_id = $ap_form->{id}
-                     WHERE id = $report_id"
-        ) or $ap_form->dberror());
+    $adjustment->rows(\@rows);
+    $adjustment->save;
 }
 
 sub _process_ar_multi {
@@ -396,23 +358,28 @@ sub _process_chart {
     my ($request, $entries) = @_;
 
     my %imported;
+    my $cfg = LedgerSMB::Company::Configuration->new(dbh => $request->{dbh});
     foreach my $entry (@$entries){
         my %settings;
 
+        ###BUG missing import settings: 'recon' and "is_temp"
         @settings{qw( accno description charttype
                       category contra tax link heading gifi_accno )} = @$entry;
-
-        my @link = split /:/, $settings{link};
-        @settings{ @link } = ( (1) x @link);
+        if ($settings{charttype} eq 'A') {
+            $settings{type} = 'account';
+        }
+        elsif ($settings{charttype} eq 'H') {
+            $settings{type} = 'heading';
+        }
+        $settings{link} = [ split /:/, $settings{link} ];
 
         die "Unable to resolve heading $settings{heading} to its id; available: " . join(' ', sort keys %imported)
             if ($settings{heading}
                 and not exists $imported{$settings{heading}});
-        $settings{heading} = $imported{$settings{heading}}->{id};
+        $settings{heading_id} = $imported{$settings{heading}}->{id};
 
-        my $account =
-            LedgerSMB::DBObject::Account->new(%settings,
-                                              dbh => $request->{dbh});
+        $settings{gifi_id} = $settings{gifi_accno};
+        my $account = $cfg->coa_nodes->create(%settings);
         $account->save();
         $imported{$settings{accno}} = $account;
     }
@@ -471,29 +438,14 @@ sub _process_timecard {
 
 sub _process_inventory {
     my ($request, $entries) = @_;
-    my $dbh = $request->{dbh};
-
-    $dbh->do( # Not worth parameterizing for one input
-              'INSERT INTO inventory_report
-                            (transdate, source)
-                     VALUES ('.$dbh->quote($request->{transdate}).
-              q{, 'CSV upload')}
-        ) or $request->dberror();
-
-    my ($report_id) = $dbh->selectrow_array(
-        q{SELECT currval('inventory_report_id_seq')}
-        ) or $request->dberror();
-
     @$entries =
         map { map_columns_into_hash($cols->{inventory}, $_) } @$entries;
 
-    return _inventory_single_date($request, $entries,
-                            $report_id, $request->{transdate});
+    return _inventory_single_date($request, $entries, $request->{transdate});
 }
 
 sub _process_inventory_multi {
     my ($request, $entries) = @_;
-    my $dbh = $request->{dbh};
 
     @$entries =
         map { map_columns_into_hash($cols->{inventory_multi}, $_) }
@@ -503,21 +455,9 @@ sub _process_inventory_multi {
         push @{$dated_entries{$entry->{date}}}, $entry;
     }
 
-    for my $key (keys %dated_entries) {
-        $dbh->do( # Not worth parameterizing for one input
-                  'INSERT INTO inventory_report
-                            (transdate, source)
-                     VALUES ('.$dbh->quote($key).
-                  q{, 'CSV upload (' || }.$dbh->quote($request->{transdate})
-                  .q{ || ')')}
-            ) or $request->dberror();
-
-        my ($report_id) = $dbh->selectrow_array(
-            q{SELECT currval('inventory_report_id_seq')}
-            ) or $request->dberror();
-
-        &_inventory_single_date($request, $dated_entries{$key},
-                                $report_id, $key);
+    for my $transdate (keys %dated_entries) {
+        &_inventory_single_date($request,
+                                $dated_entries{$transdate}, $transdate);
     }
     return;
 }
@@ -525,8 +465,7 @@ sub _process_inventory_multi {
 sub _process_parts {
     my ($request, $entries, $columns, $acc_types) = @_;
     my %table_columns =
-        map { $_ => ($_ =~ s/_accno$/_accno_id/r) }
-        map { $_ eq 'partsgroup' ? 'partsgroup_id' : $_ }
+        map { $_ => ($_ =~ s/(_accno|partsgroup)$/$1_id/r) }
         grep { $_ ne 'taxaccnos' }
         @{$columns};
     my %column_placeholders =
@@ -539,7 +478,7 @@ sub _process_parts {
                                               AND al.description = ? ))'
                   : ($_ eq 'partsgroup' ?
                      '(SELECT id FROM partsgroup WHERE partsgroup = ?)'
-                     : $_))}
+                     : '?'))}
         keys %table_columns;
 
     my $stmt =
@@ -620,7 +559,7 @@ our $process = {
     timecard        => \&_process_timecard,
     inventory       => \&_process_inventory,
     inventory_multi => \&_process_inventory_multi,
-    goods           => \&_process_goods,
+    parts           => \&_process_goods,
     services        => \&_process_services,
     overhead        => \&_process_overhead,
 };
@@ -635,17 +574,17 @@ sub _parse_file {
     my $self = shift @_;
 
     my $handle = $self->upload('import_file');
-    my $csv = Text::CSV->new;
+    my $csv = Text::CSV->new({ blank_is_undef => 1 });
     $csv->header($handle);
     $self->{import_entries} = $csv->getline_all($handle);
 
     return ([$csv->fields], @{$self->{import_entries}});
 }
 
+
+
 =head2 begin_import
-
 This displays the begin data entry screen.
-
 =cut
 
 sub begin_import {
@@ -657,16 +596,11 @@ sub begin_import {
     if (ref($template_setup->{$request->{type}}) eq 'CODE') {
         $template_setup->{$request->{type}}($request);
     }
-
-    # $request->{page_id} = $request->{type};
-    # $request->{page_id} =~ s/_/-/;
-    # $request->{page_id} .= '-import';
-    $request->{page_id} = 'batch-import';
-    my $template = LedgerSMB::Template::UI->new_UI;
-    return $template->render($request, 'import_csv/' . $template_file,
-                             { request => $request });
+    return [ 200, [ 'Content-Type' => 'application/json' ], [ '{}' ] ];
 }
 
+
+
 =head2 run_import
 
 run_import is the routine responsible for the primary work.  It accepts the
@@ -674,20 +608,29 @@ data in $request and processes it according to the dispatch tables.
 
 =cut
 
+my $json = JSON::PP->new;
+
 sub run_import {
     my ($request) = @_;
 
-    my @entries = _parse_file($request);
-    my $headers = shift @entries;
-    if (ref($preprocess->{$request->{type}}) eq 'CODE'){
-        $preprocess->{$request->{type}}($request, \@entries, $headers);
-    }
-    if ($process->{$request->{type}}($request, \@entries)){
-        if (ref($postprocess->{$request->{type}}) eq 'CODE'){
-            $postprocess->{$request->{type}}($request, \@entries);
+    try {
+        my @entries = _parse_file($request);
+        my $headers = shift @entries;
+        if (ref($preprocess->{$request->{type}}) eq 'CODE'){
+            $preprocess->{$request->{type}}($request, \@entries, $headers);
         }
+        if ($process->{$request->{type}}($request, \@entries)){
+            if (ref($postprocess->{$request->{type}}) eq 'CODE'){
+                $postprocess->{$request->{type}}($request, \@entries);
+            }
+        }
+        return begin_import($request);
+    }
+    catch ($e) {
+        return [ 500,
+                 [ 'Content-Type' => 'application/json' ],
+                 [ $json->encode({ error => "$e" }) ] ];
     }
-    return begin_import($request);
 }
 
 =head1 LICENSE AND COPYRIGHT
@@ -700,17 +643,4 @@ your software.
 
 =cut
 
-{
-    local ($!, $@) = ( undef, undef);
-    my $do_ = 'scripts/custom/import_trans.pl';
-    if ( -e $do_ ) {
-        unless ( do $do_ ) {
-            if ($! or $@) {
-                warn "\nFailed to execute $do_ ($!): $@\n";
-                die ( "Status: 500 Internal server error (import_csv.pm)\n\n" );
-            }
-        }
-    }
-};
-
 1;
