@@ -28,31 +28,17 @@ use LedgerSMB::Report::Unapproved::Batch_Detail;
 use LedgerSMB::Scripts::payment;
 use LedgerSMB::Scripts::reports;
 use LedgerSMB::Setting;
-use LedgerSMB::Sysconfig;
-use LedgerSMB::Template::UI;
 
 use LedgerSMB::old_code qw(dispatch);
 
-use File::Temp;
+use Archive::Zip qw(:CONSTANTS :ERROR_CODES);
+use File::Temp qw/ :seekable /;
 use HTTP::Status qw( HTTP_OK);
 
 
 our $VERSION = '0.1';
 our $custom_batch_types = {};
 
-{
-    local ($!, $@) = (undef, undef);
-    my $do_ = 'scripts/custom/vouchers.pl';
-    if ( -e $do_ ) {
-        unless ( do $do_ ) {
-            if ($! or $@) {
-                warn "\nFailed to execute $do_ ($!): $@\n";
-                die (  'Status: 500 Internal server error ('
-                        . __FILE__ . '.pm - ' . __LINE__ . ")\n\n" );
-            }
-        }
-    }
-}
 
 =head2 create_batch($request)
 
@@ -84,7 +70,7 @@ sub create_batch {
 
     $batch->get_search_results({mini => 1});
 
-    my $template = LedgerSMB::Template::UI->new_UI;
+    my $template = $request->{_wire}->get('ui');
     return $template->render($request, 'create_batch',
                              { request => $request,
                                batch => $batch });
@@ -168,7 +154,7 @@ C<$request> is a L<LedgerSMB> object reference, which must contain:
 
 =item * batch_type
 
-Must be one of: 
+Must be one of:
 C<ap>, C<ar>, C<gl>, C<sales_invoice>, C<vendor_invoice>, C<receipt>,
 C<payment>, C<payment_reversal>, C<receipt_reversal>.
 
@@ -264,7 +250,7 @@ sub list_batches {
     my ($request) = @_;
     $request->open_form;
     return $request->render_report(
-        LedgerSMB::Report::Unapproved::Batch_Overview->new(%$request)
+        LedgerSMB::Report::Unapproved::Batch_Overview->new(approved => 0, %$request)
         );
 }
 
@@ -286,6 +272,8 @@ sub get_batch {
     return $request->render_report(
         LedgerSMB::Report::Unapproved::Batch_Detail->new(
             default_language => $setting->get('default_language'),
+            language         => $request->{_user}->{language},
+            languages        => $request->enabled_languages,
             %$request,
         ));
 }
@@ -518,19 +506,23 @@ and gl transactions are not printed.
 
 sub print_batch {
     my ($request) = @_;
-    my $report = LedgerSMB::Report::Unapproved::Batch_Detail->new(%$request);
+    my $report = LedgerSMB::Report::Unapproved::Batch_Detail->new(
+        %$request,
+        language         => $request->{_user}->{language},
+        languages        => $request->enabled_languages,
+        );
     $request->{format} = 'pdf';
     $request->{media} = 'zip';
 
     # Make sure we have a temporary directory which gets cleaned up
     # after exiting this routine
-    my $dir = File::Temp->newdir( CLEANUP => 1);
+    my $dir = File::Temp->newdir( CLEANUP => 1 );
     my $dirname = $dir->dirname;
 
     # zipdir gets consumed by io.pl and arapprn.pl
     $request->{zipdir} = $dirname;
 
-    $report->run_report;
+    $report->run_report($request);
 
     my @files =
         map {
@@ -545,25 +537,24 @@ sub print_batch {
                     $_,
                     $request
                 );
-                return 1;
             }
-            return ();
+            $entry ? 1 : ();
         }
         @{$report->rows};
 
     if (@files) {
-        my $zipcmd = $LedgerSMB::Sysconfig::zip;
-        $zipcmd =~ s/\%dir/$dirname/g;
-        `$zipcmd`;
-
-        my $file_path = "$dirname.zip";
+        my $zip = Archive::Zip->new;
+        unless ( $zip->addTree("$dirname/.", '') == AZ_OK ) {
+            die 'Unable to add vouchers from temporary directory to zip file';
+        };
+        my $fh = File::Temp->new( CLEANUP => 1 );
+        unless ( $zip->writeToFileHandle( $fh, 1 ) == AZ_OK ) {
+            die 'Unable to write voucher zip output to temporary file';
+        }
+        $fh->seek( 0, 0 );
 
         return sub {
             my $responder = shift;
-
-            open my $zip, '<:bytes', $file_path
-                or die "Failed to open temporary zip file $file_path : $!";
-
             $responder->(
                 [
                  HTTP_OK,
@@ -573,13 +564,8 @@ sub print_batch {
                       'attachment; filename="batch-'
                       . $request->{batch_id} . '.zip"',
                  ],
-                 $zip   # the file-handle
+                 $fh   # the file-handle
                 ]);
-
-            close $zip
-                or warn "Failed to close temporary zip file $file_path : $!";
-            unlink $file_path
-                or warn "Failed to unlink temporary zip file $file_path : $!";
         };
     }
     else {
@@ -611,7 +597,7 @@ maps to the selection stored proc
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright (C) 2009-2018 The LedgerSMB Core Team
+Copyright (C) 2009-2022 The LedgerSMB Core Team
 
 This file is licensed under the GNU General Public License version 2, or at your
 option any later version.  A copy of the license should have been included with
