@@ -1323,6 +1323,16 @@ sub approve {
     my $wf = $form->{_wire}->get('workflows')
         ->fetch_workflow( 'AR/AP', $form->{workflow_id} );
     $wf->execute_action( $form->{__action} );
+    my $query =
+        ($form->{vc} eq 'customer')
+        ? 'select invnumber from ar where id = ?'
+        : 'select invnumber from ap where id = ?';
+    my $sth = $form->{dbh}->prepare($query)
+        or $form->dberror($query);
+    $sth->execute( $form->{id} )
+        or $form->dberror($query);
+    ($form->{invnumber}) = $sth->fetchrow_array;
+    $form->dberror($query) if $sth->err;
 
     if ($form->{callback}){
         print "Location: $form->{callback}\n";
