@@ -180,7 +180,7 @@ sub select_name {
     print qq|
 <body class="lsmb">
 
-<form method="post" data-dojo-type="lsmb/Form" action=$form->{script}>
+<form method="post" data-dojo-type="lsmb/Form" action="$form->{script}">
 
 <table width=100%>
   <tr>
@@ -251,7 +251,7 @@ qq|<td><input name="new_meta_number_$i" type=hidden value="$ref->{meta_number}">
     # delete variables
     for (qw(nextsub name_list)) { delete $form->{$_} }
 
-    $form->{action} = "name_selected";
+    $form->{__action} = "name_selected";
 
     $form->hide_form;
 
@@ -259,7 +259,7 @@ qq|<td><input name="new_meta_number_$i" type=hidden value="$ref->{meta_number}">
 <input type=hidden name=nextsub value=name_selected>
 <input type=hidden name=vc value="$table">
 <br>
-<button data-dojo-type="dijit/form/Button" class="submit" type="submit" name="action" value="continue">|
+<button data-dojo-type="dijit/form/Button" class="submit" type="submit" name="__action" value="continue">|
       . $locale->text('Continue')
       . qq|</button>
 </form>
@@ -304,7 +304,7 @@ sub add_transaction {
     my ($module) = @_;
 
     delete $form->{script};
-    $form->{action} = "add";
+    $form->{__action} = "add";
     $form->{type} = "invoice" if $module =~ /(is|ir)/;
 
     $form->{callback} = $form->escape( $form->{callback}, 1 );
@@ -317,19 +317,31 @@ sub add_transaction {
 
 }
 
-
 sub post_as_new {
+    my %args = @_;
+
+    $form->{old_workflow_id} = $form->{workflow_id};
+    for (qw(id printed emailed workflow_id invnumber)) { delete $form->{$_} }
+    $form->{invnumber} = $args{invnumber} // '';
+
+    my $wf = $form->{_wire}->get('workflows')
+        ->create_workflow( 'AR/AP' );
+    $form->{workflow_id} = $wf->id;
+    $form->{__action} = 'post';
 
-    for (qw(id printed emailed queued)) { delete $form->{$_} }
     &post;
-
 }
 
 sub print_and_post_as_new {
 
-    for (qw(id printed emailed queued)) { delete $form->{$_} }
-    &print_and_post;
+    $form->{old_workflow_id} = $form->{workflow_id};
+    for (qw(id printed emailed workflow_id)) { delete $form->{$_} }
 
+    my $wf = $form->{_wire}->get('workflows')
+        ->create_workflow( 'AR/AP' );
+    $form->{workflow_id} = $wf->id;
+
+    &print_and_post;
 }
 
 sub repost {
@@ -338,6 +350,10 @@ sub repost {
 
 sub schedule {
 
+    if ($form->{formname} =~ m/invoice/) {
+        &prepare_invoice;
+        $form->generate_selects;
+    }
     (
         $form->{recurringreference}, $form->{recurringstartdate},
         $form->{recurringrepeat},    $form->{recurringunit},
@@ -440,13 +456,13 @@ sub schedule {
 
     }
 
-    if (   %{LedgerSMB::Sysconfig::printer}
-        && ${LedgerSMB::Sysconfig::latex}
+    if ( $form->{_wire}->get( 'printers' )->names # any printers ?
         && %formname )
     {
-        $selectprinter = qq|<option>\n|;
-        for ( sort keys %{LedgerSMB::Sysconfig::printer} ) {
-            $selectprinter .= qq|<option value="$_">$_\n|;
+        $selectprinter = qq|<option></option>\n|;
+        for ( sort $form->{_wire}->get( 'printers' )->names ) {
+            my $prn = $form->escape($_);
+            $selectprinter .= qq|<option value="$prn">$prn</option>\n|;
         }
 
         # formname:format:printer
@@ -549,7 +565,7 @@ sub schedule {
     print qq|
 <body class="lsmb">
 
-<form method="post" data-dojo-type="lsmb/Form" action=$form->{script}>
+<form method="post" data-dojo-type="lsmb/Form" action="$form->{script}">
 
 <table width=100%>
   <tr class=listtop>
@@ -625,7 +641,7 @@ sub schedule {
     }
 
     # delete variables
-    for (qw(action recurring)) { delete $form->{$_} }
+    for (qw(recurring)) { delete $form->{$_} }
     for (
         qw(reference startdate nextdate enddate repeat unit howmany payment print email message)
       )
@@ -735,11 +751,11 @@ sub reprint {
             $form->{rowcount}--;
             for ( 1 .. $form->{rowcount} ) {
                 $form->{"amount_$_"} =
-                  $form->format_amount( \%myconfig, $form->{"amount_$_"}, 2 );
+                  $form->format_amount( \%myconfig, $form->{"amount_$_"}, LedgerSMB::Setting->new(%$form)->get('decimal_places') );
             }
             for ( split / /, $form->{taxaccounts} ) {
                 $form->{"tax_$_"} =
-                  $form->format_amount( \%myconfig, $form->{"tax_$_"}, 2 );
+                  $form->format_amount( \%myconfig, $form->{"tax_$_"}, LedgerSMB::Setting->new(%$form)->get('decimal_places') );
             }
             $pf = "print_transaction";
         }
@@ -759,39 +775,14 @@ sub reprint {
 
     for ( 1 .. $form->{paidaccounts} ) {
         $form->{"paid_$_"} =
-          $form->format_amount( \%myconfig, $form->{"paid_$_"}, 2 );
+          $form->format_amount( \%myconfig, $form->{"paid_$_"}, LedgerSMB::Setting->new(%$form)->get('decimal_places') );
     }
 
     $form->{copies} = 1;
 
     &$pf();
 
-    if ( $form->{media} eq 'email' ) {
-
-        # add email message
-        $now = scalar localtime;
-        $cc  = $locale->text( 'Cc: [_1]', $form->{cc} ) . qq|\n| if $form->{cc};
-        $bcc = $locale->text( 'Bcc: [_1]', $form->{bcc} ) . qq|\n|
-          if $form->{bcc};
-
-        $form->{intnotes} .= qq|\n\n| if $form->{intnotes};
-        $form->{intnotes} .=
-            qq|[email]\n|
-          . $locale->text( 'Date: [_1]', $now ) . qq|\n|
-          . $locale->text( 'To: [_1]',   $form->{email} )
-          . qq|\n${cc}${bcc}|
-          . $locale->text( 'Subject: [_1]', $form->{subject} )
-          . qq|\n\n|
-          . $locale->text('Message: ');
-
-        $form->{intnotes} .=
-          ( $form->{message} ) ? $form->{message} : $locale->text('sent');
-
-        $form->save_intnotes( \%myconfig, $form->{module} );
-
-        0; #SC: Reversing the expected value to work with its caller
-    }
-
+    0;
 }
 
 
