@@ -137,7 +137,17 @@ sub approve {
 
     my $wf = $form->{_wire}->get('workflows')
         ->fetch_workflow( 'AR/AP', $form->{workflow_id} );
-    $wf->execute_action( 'approve' ) if $wf;
+    $wf->execute_action( 'approve' );
+    my $query =
+        ($form->{vc} eq 'customer')
+        ? 'select invnumber from ar where id = ?'
+        : 'select invnumber from ap where id = ?';
+    my $sth = $form->{dbh}->prepare($query)
+        or $form->dberror($query);
+    $sth->execute( $form->{id} )
+        or $form->dberror($query);
+    ($form->{invnumber}) = $sth->fetchrow_array;
+    $form->dberror($query) if $sth->err;
     edit();
 }
 
@@ -1503,12 +1513,6 @@ sub print_form {
 
             $trans_wf = $form->{_wire}->get('workflows')
                 ->fetch_workflow( 'AR/AP', $wf_id );
-            if (grep { $_ eq 'save' } $trans_wf->get_current_actions) {
-                $trans_wf->execute_action( 'save' );
-            }
-            if (grep { $_ eq 'post' } $trans_wf->get_current_actions) {
-                $trans_wf->execute_action( 'post' );
-            }
         }
 
         $trans_wf->context->param( '_email_data' => $email_data );
