@@ -62,26 +62,26 @@ use open ':utf8';
 use utf8;
 
 
+use LedgerSMB;
 use LedgerSMB::App_State;
 use LedgerSMB::Company_Config;
+use LedgerSMB::Magic qw( SCRIPT_OLDSCRIPTS );
 use LedgerSMB::PGNumber;
-use LedgerSMB::Sysconfig;
 use LedgerSMB::Setting::Sequence;
 use LedgerSMB::Setting;
 
 
 use Carp;
-use DBI;
 use List::Util qw(first);
-use Log::Log4perl;
+use Log::Any;
 use LWP::Simple;
+use PGObject;
 use Symbol;
 use Time::Local;
-use Try::Tiny;
 
 
 
-our $logger = Log::Log4perl->get_logger('LedgerSMB::Form');
+our $logger = Log::Any->get_logger(category => 'LedgerSMB::Form');
 
 
 =item new Form([$argstr])
@@ -104,22 +104,21 @@ sub new {
 
     my %orig;
     if (defined $argstr) {
-        %orig = split( /[&=]/, $argstr);
+        %orig = split( /[&=]/, $argstr, -1);
         for ( keys %orig ) {
             $self->{unescape( "", $_) } = unescape( "", $orig{$_} );
         }
-        delete $self->{header};
 
         for my $p(keys %$self){
             utf8::decode($self->{$p});
             utf8::upgrade($self->{$p});
+            delete $self->{$p} if $self->{$p} eq '_!lsmb!empty!_';
             $self->{$p} =~ s/\N{NULL}//g;
         }
         $self->{nextsub} //= '';
-        $self->{action} //= $self->{nextsub};
+        $self->{__action} //= $self->{nextsub};
     }
-    $self->{version}   = '1.8.0';
-    $self->{dbversion} = '1.8.0';
+    $self->{version} = $self->{dbversion} = $LedgerSMB::VERSION;
 
 
     $self;
@@ -160,8 +159,10 @@ sub check_form {
     my ($self) = @_;
 
     return 0 unless $self->{form_id};
-    my $sth = $self->{dbh}->prepare('select form_check(?, ?)');
-    $sth->execute($self->{session_id}, $self->{form_id});
+    my $sth = $self->{dbh}->prepare('select form_check(?, ?)')
+        or die $self->{dbh}->errstr;
+    $sth->execute($self->{session_id}, $self->{form_id})
+        or die $self->{dbh}->errstr;
     my @results = $sth->fetchrow_array();
     return $results[0];
 }
@@ -172,8 +173,10 @@ sub close_form {
         delete $self->{form_id};
     }
 
-    my $sth = $self->{dbh}->prepare('select form_close(?, ?)');
-    $sth->execute($self->{session_id}, $self->{form_id});
+    my $sth = $self->{dbh}->prepare('select form_close(?, ?)')
+        or die $self->{dbh}->errstr;
+    $sth->execute($self->{session_id}, $self->{form_id})
+        or die $self->{dbh}->errstr;
     my @results = $sth->fetchrow_array();
     delete $self->{close_form};
     return $results[0];
@@ -343,6 +346,20 @@ sub error {
     Carp::croak $msg;
 }
 
+=item $form->call_procedure()
+
+=cut
+
+sub call_procedure {
+    my $self = shift;
+    my %args = @_;
+    $args{funcschema} ||= $self->{_wire}->get( 'db' )->schema;
+    $args{funcname} ||= $args{procname};
+    $args{dbh} = $self->{dbh};
+    $args{args} ||= [];
+    return PGObject->call_procedure(%args);
+}
+
 =item $form->finalize_request();
 
 Stops further processing, allowing post-request cleanup on intermediate
@@ -353,6 +370,7 @@ This function replaces explicit 'exit()' calls.
 =cut
 
 sub finalize_request {
+    undef $@;
     die;
 }
 
@@ -410,14 +428,14 @@ sub numtextrows {
 
 =item $form->dberror($msg);
 
-Outputs a message as in $form->error but with $DBI::errstr automatically
+Outputs a message as in $form->error but with $form->{dbh}->errstr automatically
 appended to $msg.
 
 =cut
 
 sub dberror {
     my ( $self, $msg ) = @_;
-    $self->error( "$msg\n" . $DBI::errstr );
+    $self->error( "$msg\n" . $self->{dbh}->errstr );
 }
 
 =item $form->isblank($name, $msg);
@@ -462,37 +480,17 @@ sub header {
     $self->{header} = 1;
 }
 
-=item $form->open_status_div
+=item $form->open_status_div( $div_id )
 
-Returns a div tag with an id of statusdiv.
-
-If $form->{id} is set and $form->{approved} the class is set to "posted" and if
-id is set but not approved, this is set to "saved."  If neither applies, we set
-to "new."
+Returns a div tag with an id of C<$div_id>.
 
 =cut
 
 sub open_status_div {
     my ($self, $div_id) = @_;
-    my $class;
-    if ($self->{approved} and $self->{id}){
-        $class = "posted";
-    }
-    elsif ($self->{id}){
-        $class = "saved";
-    }
-    else {
-        $class = "new";
-    }
 
-    my $status = $self->{_locale}->text(
-        'Action: [_1], ID: [_2]',
-        $self->{action},
-        $self->{id}
-        );
     my $id = $div_id ? "id=\"$div_id\"" : '';
-    return "<div $id class=\"$class\">
-            <div id=\"history\">$status</div>";
+    return "<div $id>";
 }
 
 =item $form->close_status_div
@@ -523,6 +521,8 @@ sub redirect {
     }
     else {
         $self->info($msg);
+        print '</body>';
+        $self->finalize_request;
     }
 }
 
@@ -538,27 +538,18 @@ sub _redirect {
         return;
     }
 
-    if (first { $_ eq $script } @{LedgerSMB::Sysconfig::newscripts}) {
+    unless (first { $_ eq $script } SCRIPT_OLDSCRIPTS->@*) {
         print "Location: $self->{callback}\n";
         print "Content-type: text/html\n\n";
         return;
     }
 
-    $self->error(
-        $self->{_locale}->text(
-            "[_1]:[_2]:[_3]: Invalid Redirect",
-            __FILE__,
-            __LINE__,
-           $script
-        )
-    ) unless first { $_ eq $script } @{LedgerSMB::Sysconfig::scripts};
-
-
     my $form = Form->new($argv);
-    $form->{$_} = $self->{$_} for qw(
-      dbh login favicon stylesheet titlebar password vc header _auth _locale
-    );
-    $form->{action} ||= $self->{action}; # default to old action if not set
+    $form->{$_} = $self->{$_} for (
+        qw( dbh login favicon stylesheet titlebar password vc header ),
+        grep { /^_/ and $_ ne '__action' } keys %$self
+        );
+    $form->{__action} ||= $self->{__action}; # default to old action if not set
     $form->{script} = $script;
 
     my %myconfig = %{ LedgerSMB::User->fetch_config( $form ) };
@@ -569,7 +560,7 @@ sub _redirect {
     $lsmb_legacy::form = $form;
     require "old/bin/$script";
 
-    my $ref = qualify_to_ref $form->{action}, 'lsmb_legacy';
+    my $ref = qualify_to_ref $form->{__action}, 'lsmb_legacy';
     &{ *{$ref} };
 
 }
@@ -1004,8 +995,7 @@ about buttons, $name is the key for the element in $button to output.  Each
 value in $button is a reference to a hash of two elements, 'key' and 'value'.
 
 $name is the value of the button that gets sent to the server when clicked,
-$button->{$name}{key} is the accesskey, and $button->{$name}{value} is the label
-for the button.
+$button->{$name}{value} is the label for the button.
 
 =cut
 
@@ -1016,8 +1006,17 @@ sub print_button {
 
     my $type = $button->{$name}{type} // 'dijit/form/Button';
     $btn++;
+
+    my $doing_toast =
+        $button->{$name}{doing} ? qq|data-lsmb-doing="$button->{$name}{doing}"|
+        : '';
+    my $done_toast =
+        $button->{$name}{done} ? qq|data-lsmb-done="$button->{$name}{done}"|
+        : '';
+
+    my $title = $button->{$name}{tooltip} || $button->{$name}{value};
     print
-qq|<button data-dojo-type="$type" class="submit" type="submit" name="action" value="$name" id="action-$name-$btn" accesskey="$button->{$name}{key}" title="$button->{$name}{value} [Alt-$button->{$name}{key}]">$button->{$name}{value}</button>\n|;
+qq|<button data-dojo-type="$type" class="submit" type="submit" name="__action" value="$name" id="action-$name-$btn" title="$title" $doing_toast $done_toast>$button->{$name}{value}</button>\n|;
 }
 
 
@@ -1063,28 +1062,6 @@ sub generate_selects {
         }
     }
 
-     # projects
-    if ( $form->{all_project} && @{ $form->{all_project} } ) {
-        $form->{selectprojectnumber} = "<option></option>\n";
-          $form->{selectprojectnumber} = "";
-
-        for ( @{ $form->{all_project} } ) {
-            my $value = "$_->{projectnumber}--$_->{id}";
-            $form->{selectprojectnumber} .=
-                     # change the format here, then change it below!
-                     qq|<option value="$value">$_->{projectnumber}</option>\n|;
-        }
-
-          if ($form->{rowcount}) {
-                for my $i ( 1 .. $form->{rowcount} ) {
-                     $form->{"selectprojectnumber_$i"} =
-                          $form->{"selectprojectnumber"};
-                     $form->{"selectprojectnumber_$i"} =~
-                          s/(value="\Q$form->{"projectnumber_$i"}\E")/$1 selected="selected"/;
-                }
-          }
-    }
-
     # departments
     if ( $form->{all_department} && @{ $form->{all_department} } ) {
         $form->{selectdepartment} = "<option></option>\n";
@@ -1129,7 +1106,7 @@ sub generate_selects {
               my $vc = $form->{vc};
               my $search_value = $form->{$vc};
               $search_value .= qq|--$form->{"${vc}_id"}|
-                  unless $search_value =~ /--/;
+                  unless $search_value and $search_value =~ /--/;
 
                 for ( @{ $form->{"all_$form->{vc}"} } ) {
                      my $value = "$_->{name}--$_->{id}";
@@ -1163,27 +1140,28 @@ sub generate_selects {
                 }
           }
 
-        my $min_lines = $LedgerSMB::Company_Config::settings->{min_empty} // 0;
-        my $rowcount = $form->{rowcount} + $min_lines;
+        my $min_lines = $form->get_setting('min_empty') // 0;
+        my $rowcount = ($form->{rowcount}//0) + $min_lines;
         if ($rowcount) {
                 for my $i ( 1 .. $rowcount ) {
                      $form->{"select$form->{ARAP}_amount_$i"} =
-                          $form->{"select$form->{ARAP}_amount"};
-                     $form->{"select$form->{ARAP}_amount_$i"} =~
-                          s/(value="\Q$form->{"$form->{ARAP}_amount_$i"}\E")/$1 selected="selected"/;
+                         $form->{"select$form->{ARAP}_amount"};
+                     if ($form->{"$form->{ARAP}_amount_$i"}) {
+                         $form->{"select$form->{ARAP}_amount_$i"} =~
+                             s/(value="\Q$form->{"$form->{ARAP}_amount_$i"}\E")/$1 selected="selected"/;
+                     }
                 }
           }
      }
 
      # formats
-    $form->{selectformat} = qq|<option value="html">html<option value="csv">csv\n|;
-    if ( ${LedgerSMB::Sysconfig::latex} ) {
-        $form->{selectformat} .= qq|
-            <option value="postscript">|
-                . $form->{_locale}->text('Postscript')
-                . qq|<option value="pdf">|
-                . $form->{_locale}->text('PDF');
-    }
+    $form->{selectformat} =
+        join('',
+             map {
+                 my $val = lc $_;
+                 qq|<option value="$val">$_|
+             } $form->{_wire}->get( 'output_formatter' )->get_formats
+        );
 
     # warehouse
     if ( $form->{all_warehouse} &&  @{ $form->{all_warehouse} } ) {
@@ -1207,13 +1185,7 @@ Returns true if images should get be retrieved for embedding in templates
 
 sub test_should_get_images {
     my ($self)  = @_;
-    my $dbh = $self->{dbh};
-    my $sth = $dbh->prepare(
-        "SELECT value FROM defaults WHERE setting_key = 'template_images'"
-    );
-    $sth->execute;
-    my ($retval) = $sth->fetchrow_array();
-    return $retval;
+    return $self->get_setting( 'template_images' );
 }
 
 
@@ -1234,13 +1206,14 @@ sub db_init {
     $self->{dbh} = $dbh;
     _set_datestyle($dbh);
 
-    LedgerSMB::Company_Config::initialize($self);
+    LedgerSMB::Company_Config::initialize($self->{dbh});
 }
 
 sub _set_datestyle {
     my $dbh = shift;
-    my $datequery = 'select dateformat from user_preference join users using(id)
-                      where username = CURRENT_USER';
+    my $datequery =
+        q{select "value" from user_preference p join users u on u.id = p.user_id
+           where "name" = 'dateformat' and username = CURRENT_USER};
     my $date_sth = $dbh->prepare($datequery);
     $date_sth->execute;
     my ($datestyle) = $date_sth->fetchrow_array;
@@ -1251,7 +1224,8 @@ sub _set_datestyle {
         'dd-mm-yyyy' => 'set DateStyle to \'POSTGRES, EUROPEAN\'',
         'dd.mm.yyyy' => 'set DateStyle to \'GERMAN\''
     );
-    $dbh->do( $date_query{ $datestyle } ) if $date_query{ $datestyle };
+    $dbh->do( $date_query{ $datestyle } )
+        if $datestyle and $date_query{ $datestyle };
     return;
 }
 
@@ -1266,31 +1240,11 @@ Returns true if any roles are allowed, false otherwise.
 sub is_allowed_role {
     my ($self, $rolelist) = @_;
     my $sth = $self->{dbh}->prepare('SELECT lsmb__is_allowed_role(?)');
-    $sth->execute($rolelist) || die $DBI::errstr;
+    $sth->execute($rolelist) || die $sth->errstr;
     my ($access) = $sth->fetchrow_array;
     return $access;
 }
 
-=item $form->dbquote($var);
-
-If $var is an empty string, return NULL, otherwise return $var as quoted by
-$form->{dbh}->quote($var).
-
-=cut
-
-sub dbquote {
-
-    my ( $self, $var ) = @_;
-
-    if ( $var eq '' ) {
-        $_ = "NULL";
-    }
-    else {
-        $_ = $self->{dbh}->quote($var);
-    }
-    $_;
-}
-
 =item $form->update_balance($dbh, $table, $field, $where, $value);
 
 B<WARNING>: This is a dangerous private function.  All apps calling it must be
@@ -1342,7 +1296,7 @@ If $is_oe is true, the value of trans_id is NULL and of oe_id is $id.
 sub add_shipto {
 
     my ($self, $id, $is_oe) = @_;
-        if (! $self->{locationid}) {
+    if (! $self->{shiptolocationid}) {
         return;
     }
 
@@ -1368,7 +1322,7 @@ sub add_shipto {
         $sth->execute(
                         $trans_id,
             $oe_id,
-            $self->{locationid}
+            $self->{shiptolocationid}
               ) || $self->dberror($query);
 
     $sth->finish;
@@ -1476,20 +1430,27 @@ sub get_name {
     my $query = qq/
         SELECT c.*, coalesce(ecl.address, el.address) as address,
                        coalesce(ecl.city, el.city) as city,
+                       coalesce(ecl.zipcode, el.zipcode) as zipcode,
+                       coalesce(ecl.state, el.state) as state,
+                       coalesce(ecl.country, el.country) as country,
                        e.name, e.control_code,
                        ctf.default_reportable
                   FROM entity_credit_account c
           JOIN entity e ON (c.entity_id = e.id)
              LEFT JOIN (SELECT coalesce(line_one, '')
                                || ' ' || coalesce(line_two, '') as address,
-                               l.city, etl.credit_id
+                               l.city, etl.credit_id, mail_code as zipcode,
+                               state, (select short_name from country
+                                        where id=l.country_id) as country
                           FROM eca_to_location etl
                           JOIN location l ON etl.location_id = l.id
                           WHERE etl.location_class = 1) ecl
                         ON (c.id = ecl.credit_id)
              LEFT JOIN (SELECT coalesce(line_one, '')
                                || ' ' || coalesce(line_two, '') as address,
-                               l.city, etl.entity_id
+                               l.city, etl.entity_id, mail_code as zipcode,
+                               state, (select short_name from country
+                                        where id=l.country_id) as country
                           FROM entity_to_location etl
                           JOIN location l ON etl.location_id = l.id
                           WHERE etl.location_class = 1) el
@@ -1571,7 +1532,8 @@ sub all_vc {
         push (@queryargs2, $transdate, $transdate);
     }
     else {
-        $where = " true";
+        $where = " ec.entity_class = ?";
+        push @queryargs, $self->{vc_class};
     }
 
     $sth = $dbh->prepare($query);
@@ -1583,7 +1545,7 @@ sub all_vc {
     $sth->finish;
 
     if ( $count < ($myconfig->{vclimit} // 0) ) {
-
+        $self->{"${vc}_id"} //= 0;
         $self->{"${vc}_id"} *= 1;
 
         $query = qq|SELECT ec.id, e.name
@@ -1668,33 +1630,12 @@ sub get_regular_metadata {
     my $dbh = $self->{dbh};
     local $@;
     $transdate = $transdate->to_db if eval { $transdate->can('to_db') };
-
     $self->all_employees( $myconfig, $dbh, $transdate, 1 );
     $self->all_business_units( $transdate, $self->{"${vc}_id"} );
     $self->all_taxaccounts( $myconfig, $dbh, $transdate );
     $self->all_languages();
 }
 
-=item $form->all_accounts()
-
-Sets $form->{accounts} to all accounts.  Returns the list as well.
-Example:  my @account_list = $form->all_accounts();
-
-=cut
-
-sub all_accounts {
-    my ($self) = @_;
-    my $ref;
-    $self->{all_accounts} = [];
-    my $sth = $self->{dbh}->prepare('SELECT * FROM chart_list_all()');
-    $sth->execute || $self->dberror('SELECT * FROM chart_list_all()');
-    while ($ref = $sth->fetchrow_hashref('NAME_lc')){
-        push(@{$self->{all_accounts}}, $ref);
-    }
-    $sth->finish;
-    return @{$self->{all_accounts}};
-}
-
 =item $form->all_taxaccounts($myconfig, $dbh2[, $transdate]);
 
 Get the tax rates and numbers for all the taxes in $form->{taxaccounts}.  Does
@@ -1715,7 +1656,7 @@ sub all_taxaccounts {
 
     my $sth;
     my $query;
-    my $where;
+    my $where = '';
 
     my @queryargs = ();
 
@@ -1780,7 +1721,7 @@ sub all_employees {
     }
 
     if ($sales) {
-        $query .= qq| sales = '1' AND|;
+        $query .= qq| sales AND|;
     }
 
     $query =~ s/(WHERE|AND)$//;
@@ -1788,6 +1729,7 @@ sub all_employees {
     my $sth = $dbh->prepare($query);
     $sth->execute(@whereargs) || $self->dberror($query);
 
+    $self->{all_employee} = [];
     while ( my $ref = $sth->fetchrow_hashref('NAME_lc') ) {
         push @{ $self->{all_employee} }, $ref;
     }
@@ -1805,26 +1747,22 @@ units assigned to other customers.
 =cut
 
 sub all_business_units {
-
     my ( $self, $transdate, $credit_id, $module_name) = @_;
     $self->{bu_class} = [];
     $self->{b_units} = {};
 
     my $dbh       = $self->{dbh};
-    my $class_sth = $dbh->prepare(
-                q|SELECT * FROM business_unit__list_classes('1', ?)|
-    );
-    $class_sth->execute($module_name)
-        || $self->dberror(q|SELECT * FROM business_unit__list_classes('1', ?)|);
+    my $query     = q|SELECT * FROM business_unit__list_classes('1', ?)|;
+    my $class_sth = $dbh->prepare($query) || $self->dberror($query);
+    $class_sth->execute($module_name) || $self->dberror($query);
 
-    my $bu_sth    = $dbh->prepare(
-                q|SELECT * FROM business_unit__list_by_class(?, ?, ?, 'false')|
-    );
+    $query        = q|SELECT * FROM business_unit__list_by_class(?, ?, ?, 'false')|;
+    my $bu_sth    = $dbh->prepare($query) || $self->dberror($query);
 
+    $transdate  ||= undef; # set '' to undef
     while (my $classref = $class_sth->fetchrow_hashref('NAME_lc')){
         push @{$self->{bu_class}}, $classref;
-        $bu_sth->execute($classref->{id}, $transdate, $credit_id)
-            || $self->dberror(q|SELECT * FROM business_unit__list_by_class(?, ?, ?, 'false')|);
+        $bu_sth->execute($classref->{id}, $transdate, $credit_id) || $self->dberror($query);
         $self->{b_units}->{$classref->{id}} = [];
         while (my $buref = $bu_sth->fetchrow_hashref('NAME_lc')){
            push @{$self->{b_units}->{$classref->{id}}}, $buref;
@@ -1917,12 +1855,12 @@ not contain 'tax', the account number is appended to the space separated list
 $form->{accounts}.  $module is typically 'AR' or 'AP' and is the base type of
 the accounts looked up.
 
-If $form->{id} is not set, check $form->{"$form->{vc}_id"}.  If neither is set,
-use $form->lastname_used to populate the details.  If $form->{id} is set,
+If $form->{id} is not set, check $form->{"$form->{vc}_id"}.
+If $form->{id} is set,
 populate the invnumber, transdate, ${vc}_id, datepaid, duedate, ordnumber,
 taxincluded, currency, notes, intnotes, ${vc}, department_id, department,
 oldinvtotal, employee_id, employee, language_code, ponumber,
-reverse, printed, emailed, queued, recurring, exchangerate, and acc_trans
+reverse, printed, emailed, recurring, exchangerate, and acc_trans
 attributes of $form with details about the transaction $form->{id}.  All of
 these attributes, save for acc_trans, are scalar; $form->{acc_trans} refers to
 a hash keyed by link elements whose values are lists of references to hashes
@@ -1930,7 +1868,7 @@ describing acc_trans table entries corresponding to the transaction $form->{id}.
 The elements in the acc_trans entry hashes are accno, description, source,
 amount, memo, transdate, cleared, project_id, projectnumber, and exchangerate.
 
-The closedto, separate_duties, revtrans, and currencies $form attributes are filled with values
+The separate_duties and currencies $form attributes are filled with values
 from the defaults table, while $form->{current_date} is populated with the
 current date.  If $form->{id} is not set, then $form->{transdate} also takes on
 the current date.
@@ -1976,7 +1914,7 @@ sub create_links {
     }
 
     # now get the account numbers
-    $query = qq|SELECT a.accno, a.description, as_array(l.description) as link
+    $query = qq|SELECT a.accno, a.description, array_agg(l.description) as link
                   FROM account a
                   JOIN account_link l ON a.id = l.account_id AND NOT a.obsolete
                  WHERE (l.description LIKE ? OR a.tax)
@@ -1994,14 +1932,14 @@ sub create_links {
     $self->{accounts} = "";
 
     while ( my $ref = $sth->fetchrow_hashref('NAME_lc') ) {
+        $self->{_accno_descriptions}->{$ref->{accno}} = $ref->{description};
         my $link = $ref->{link};
 
-        push(@$link,"${module}_tax") if $tax_accounts{$ref->{accno}};
+        push(@$link,"${module}_tax") # there's no "${module}_tax" link; only a tax boolean
+            if $tax_accounts{$ref->{accno}};
 
         foreach my $key ( @$link ) {
-
-            if ( $key =~ /$module/ ) {
-
+            if ( $key =~ /$module/ ) { # *all* tax accounts selected in query above
                 # cross reference for keys
                 $xkeyref{ $ref->{accno} } = $key;
 
@@ -2016,7 +1954,6 @@ sub create_links {
             }
         }
     }
-
     $sth->finish;
 
     my $arap = ( $vc eq 'customer' ) ? 'ar' : 'ap';
@@ -2032,15 +1969,18 @@ sub create_links {
                 a.duedate, a.ordnumber,
                 a.taxincluded, a.curr AS currency, a.notes,
                 a.intnotes, ce.name AS $vc,
-            a.amount_tc AS oldinvtotal,
-            case when a.amount_tc = 0 then 0
-            else a.amount_bc / a.amount_tc end as exchangerate,
-                a.person_id, e.name AS employee,
+                a.amount_tc AS oldinvtotal,
+                case when a.amount_tc = 0 then 0
+                else a.amount_bc / a.amount_tc end as exchangerate,
+                a.person_id as employee_id, e.name AS employee,
                 c.language_code, a.ponumber, a.reverse,
-                                a.approved, ctf.default_reportable,
-                                a.description, a.on_hold, a.crdate,
-                                ns.location_id as locationid, a.is_return, $seq
+                a.approved, ctf.default_reportable,
+                a.description, a.on_hold, a.crdate,
+                ns.location_id as locationid, a.is_return, $seq,
+                t.workflow_id, t.reversing, t.reversing_reference,
+                t.reversed_by, t.reversed_by_reference
             FROM $arap a
+            JOIN transactions_reversal t ON t.id = a.id
             JOIN entity_credit_account c
                 ON (a.entity_credit_account = c.id)
             JOIN entity ce ON (ce.id = c.entity_id)
@@ -2053,7 +1993,7 @@ sub create_links {
             WHERE a.id = ? AND c.entity_class =
                 (select id FROM entity_class
                 WHERE class ilike ?)|;
-        $sth = $dbh->prepare($query);
+        $sth = $dbh->prepare($query) || $self->dberror($query);
         $sth->execute( $self->{id}, $self->{vc} ) || $self->dberror($query);
 
         $ref = $sth->fetchrow_hashref('NAME_lc');
@@ -2064,14 +2004,15 @@ sub create_links {
         }
 
         foreach my $key (keys %$ref) {
-            $self->{$key} = $ref->{$key};
+            $self->{$key} = $ref->{$key} unless defined $self->{$key};
+
         }
 
         $sth->finish;
 
         # get printed, emailed
         $query = qq|
-            SELECT s.printed, s.emailed, s.spoolfile, s.formname
+            SELECT s.printed, s.emailed, s.formname
             FROM status s WHERE s.trans_id = ?|;
         $sth = $dbh->prepare($query);
         $sth->execute( $self->{id} ) || $self->dberror($query);
@@ -2080,13 +2021,11 @@ sub create_links {
               if $ref->{printed};
             $self->{emailed} .= "$ref->{formname} "
               if $ref->{emailed};
-            $self->{queued} .= "$ref->{formname} " . "$ref->{spoolfile} "
-              if $ref->{spoolfile};
         }
         $sth->finish;
 
-        for (qw(printed emailed queued)) {
-            $self->{$_} =~ s/ +$//g
+        for (qw(printed emailed)) {
+            $self->{$_} =~ s/ +$//g if $self->{$_}
         }
 
     # get customer e-mail accounts
@@ -2117,10 +2056,9 @@ sub create_links {
     my $billing_email = 0;
 
     while ( $ref = $sth->fetchrow_hashref('NAME_lc') ) {
-        $ctype = $ref->{class_id};
-        $ctype = $id_map{$ctype};
+        $ctype = $id_map{$ref->{class_id}};
         $billing_email = 1
-        if $ref->{class_id} == 15;
+            if $ref->{class_id} == 15;
 
         # If there's an explicit billing email, don't use
         # the standard email addresses; otherwise fall back to standard
@@ -2140,8 +2078,10 @@ sub create_links {
          SELECT c.accno, c.description, a.source, a.amount_tc as amount,
                 case when a.amount_tc = 0 then 0 else (a.amount_bc/a.amount_tc)::numeric end as exchangerate,
                 a.memo,a.entry_id, a.transdate, a.cleared,
-                                compound_array(ARRAY[ARRAY[bul.class_id, bul.bu_id]])
-                                AS bu_lines
+                                array_agg(ARRAY[bul.class_id, bul.bu_id])
+                                AS bu_lines,
+               (exists (select 1 from payment_links pl
+                         where a.entry_id = pl.entry_id)) AS payment_line
             FROM acc_trans a
             JOIN account c ON (c.id = a.chart_id)
                    LEFT JOIN business_unit_ac bul ON a.entry_id = bul.entry_id
@@ -2175,20 +2115,9 @@ sub create_links {
 
         $sth->finish;
     }
-    else {
 
-        if ( !$self->{"$self->{vc}_id"} ) {
-            $self->lastname_used($vc);
-        }
-    }
-
-    for (qw(separate_duties curr closedto revtrans lock_description)) {
-        if ($_ eq 'closedto'){
-            $query = qq|
-                SELECT value::date FROM defaults
-                 WHERE setting_key = '$_'|;
-        }
-        elsif ($_ eq 'current_date') {
+    for (qw(separate_duties curr lock_description)) {
+        if ($_ eq 'current_date') {
             $query = qq| select $_|;
         }
         else {
@@ -2259,65 +2188,30 @@ sub get_setting {
     return $ref->{value};
 }
 
-=item $form->lastname_used($vc);
+=item $form->is_closed( $transdate )
 
-Fills the name, currency, ${vc}_id, duedate, and possibly invoice_notes
-attributes of $form with the last used values for the transaction type specified
-by both $vc and $form->{type}.  $vc can be either 'vendor' or 'customer' and if
-unspecified will take on the value given in $form->{vc}, defaulting to 'vendor'.
-If $form->{type} matches /_order/, the transaction type used is order, if it
-matches /_quotation/, quotations are looked through.  If $form->{type} does not
-match either of the above, then ar or ap transactions are used.
+Returns true when $transdate is in a closed period.
 
 =cut
 
-sub lastname_used {
 
-    my ($self, $vc) = @_;
-    $vc ||= $self->{vc};    # add default to correct for improper passing
-    my $arap;
-    my $where;
-    if ($vc eq 'customer') {
-        $arap = 'ar';
-    }
-    else {
-        $arap = 'ap';
-        $vc = 'vendor';
-    }
+sub is_closed {
+    my ($self, $transdate) = @_;
 
-    my $sth;
-    if ($self->{type} && $self->{type} =~ /_order/ ) {
-        $arap  = 'oe';
-        $where = "quotation = '0'";
-    }
+    return '' if not $transdate;
 
-    if ($self->{type} && $self->{type} =~ /_quotation/ ) {
-        $arap  = 'oe';
-        $where = "quotation = '1'";
-    }
+    my $query = 'select ? <= max(end_date) from account_checkpoint';
+    my $sth = $self->{dbh}->prepare($query)
+        or $self->dberror($query);
+    $transdate = $transdate->to_db() if ref $transdate;
+    $sth->execute($transdate) or $self->dberror($query);
+    my ($is_closed) = $sth->fetchrow_array;
+    die $self->dberror($query) if $sth->err != 0;
 
-    $where = "AND $where " if $where;
-    $where //= '';
-    my $query = qq|
-        SELECT entity.name, ct.curr AS currency, entity_id AS ${vc}_id,
-            current_date + ct.terms AS duedate,
-            ct.curr AS currency
-        FROM entity_credit_account ct
-        JOIN entity ON (ct.entity_id = entity.id)
-        WHERE entity.id = (select entity_id from $arap
-                            where entity_id IS NOT NULL $where
-                                 order by id DESC limit 1)|;
-
-    $sth = $self->{dbh}->prepare($query);
-    $sth->execute() || $self->dberror($query);
-
-    my $ref = $sth->fetchrow_hashref('NAME_lc');
-    for (keys %$ref) {
-        $self->{$_} = $ref->{$_}
-    }
-    $sth->finish;
+    return $is_closed;
 }
 
+
 =item $form->current_date($myconfig[, $thisdate, $days]);
 
 If $thisdate is false, get the current date from the database.
@@ -2497,7 +2391,7 @@ sub get_partsgroup {
                 t.description AS translation
             FROM partsgroup pg
             JOIN parts p ON (p.partsgroup_id = pg.id)
-            LEFT JOIN translation t ON (t.trans_id = pg.id
+            LEFT JOIN partsgroup_translation t ON (t.trans_id = pg.id
                 AND t.language_code = ?)|;
         @queryargs = (
             $p->{language_code}
@@ -2523,9 +2417,7 @@ sub get_partsgroup {
 DELETEs all status rows which have a formname of $form->{formname} and a
 trans_id of $form->{id}.  INSERTs a new row into status where trans_id is
 $form->{id}, formname is $form->{formname}, printed and emailed are true if
-their respective $form attributes match /$form->{formname}/, and spoolfile is
-the file extracted from the string $form->{queued} or NULL if there is no entry
-for $form->{formname}.
+their respective $form attributes match /$form->{formname}/.
 
 =cut
 
@@ -2538,12 +2430,6 @@ sub update_status {
 
     my $dbh = $self->{dbh};
 
-    my %queued = split / +/, $self->{queued};
-    my $spoolfile =
-      ( $queued{ $self->{formname} } )
-      ? $queued{ $self->{formname} }
-      : undef;
-
     my $query = qq|DELETE FROM status
                     WHERE formname = ?
                       AND trans_id = ?|;
@@ -2556,22 +2442,21 @@ sub update_status {
 
     $sth->finish;
 
-    return unless $self->{printed} || $self->{emailed} || $spoolfile;
+    return unless $self->{printed} || $self->{emailed};
 
     my $printed = ( $self->{printed} =~ /$self->{formname}/ ) ? "1" : "0";
     my $emailed = ( $self->{emailed} =~ /$self->{formname}/ ) ? "1" : "0";
 
     $query = qq|
         INSERT INTO status
-            (trans_id, printed, emailed, spoolfile, formname)
-        VALUES (?, ?, ?, ?, ?)|;
+            (trans_id, printed, emailed, formname)
+        VALUES (?, ?, ?, ?)|;
 
     $sth = $dbh->prepare($query);
     $sth->execute(
         $self->{id},
         $printed,
         $emailed,
-        $spoolfile,
         $self->{formname}
     ) || $self->dberror($query);
 
@@ -2581,8 +2466,8 @@ sub update_status {
 =item $form->save_status();
 
 Clears out any old status entries for $form->{id} and saves new status entries.
-Queued form names are extracted from $form->{queued}.  Printed and emailed form
-names are extracted from $form->{printed} and $form->{emailed}.  The queued,
+Printed and emailed form
+names are extracted from $form->{printed} and $form->{emailed}.  The
 printed, and emailed fields are space separated lists.
 
 =cut
@@ -2604,43 +2489,10 @@ sub save_status {
     ) || $self->dberror($query);
     $sth->finish;
 
-    my %queued;
     my $formname;
     my $printed;
     my $emailed;
 
-    if ( $self->{queued} ) {
-
-        %queued = split / +/, $self->{queued};
-
-        foreach my $formname (keys %queued) {
-
-            $printed = ( $self->{printed} =~ /$formname/ ) ? "1" : "0";
-            $emailed = ( $self->{emailed} =~ /$formname/ ) ? "1" : "0";
-
-            if ( $queued{$formname} ) {
-                $query = qq|
-                    INSERT INTO status
-                        (trans_id, printed, emailed,
-                        spoolfile, formname)
-                    VALUES (?, ?, ?, ?, ?)|;
-
-                $sth = $dbh->prepare($query);
-                $sth->execute(
-                    $self->{id},
-                    $printed,
-                    $emailed,
-                    $queued{$formname},
-                    $formname
-                ) || $self->dberror($query);
-                $sth->finish;
-            }
-
-            $formnames  =~ s/$formname//;
-            $emailforms =~ s/$formname//;
-        }
-    }
-
     # save printed, emailed info
     $formnames  =~ s/^ +//g;
     $emailforms =~ s/^ +//g;
@@ -3094,52 +2946,7 @@ sub update_defaults {
 
     my $dbh = $self->{dbh};
 
-    my $query = qq|
-        SELECT value FROM defaults
-         WHERE setting_key = ? FOR UPDATE|;
-    my $sth = $dbh->prepare($query);
-    $sth->execute($fld);
-    my ($dbvar) = $sth->fetchrow_array();
-
-    $dbvar //= "0";
-    my $var = $dbvar;
-    my $num = $dbvar;
-    my ($prefix, $suffix);
-    ($prefix, $num, $suffix) = $num =~ /(\D*)(\d+)(\D*)$/;
-
-    if ( defined $num ) {
-        my $incnum;
-
-        # if we have leading zeros check how long it is
-
-        if ( $num =~ /^0/ ) {
-            my $l = length $num;
-            $incnum = $num + 1;
-            $l -= length $incnum;
-
-            # pad it out with zeros
-            my $padzero = "0" x $l;
-            $incnum = ( "0" x $l ) . $incnum;
-        }
-        else {
-            $incnum = $num + 1;
-        }
-
-        $dbvar =~ s/\Q$prefix$num$suffix\E$/$prefix$incnum$suffix/;
-    }
-
-
-    $query = qq|
-        UPDATE defaults
-           SET value = ?
-         WHERE setting_key = ?|;
-
-    $sth = $self->{dbh}->prepare($query);
-    $sth->execute(
-        $dbvar,
-        $fld
-    ) || $self->dberror($query);
-
+    my $var = $dbh->selectrow_array('select setting_increment(?)', {}, $fld);
     return $var;
 }
 
@@ -3179,25 +2986,14 @@ If invnumber is not set, updates it.  Used when gapless numbering is in effect
 
 sub update_invnumber {
     my $self = shift;
-    my $query = 'select invnumber from ar where id = ?';
-    my $sth = $self->{dbh}->prepare($query)
+    my $query =
+        q|update ar
+             set invnumber = setting_increment('sinumber')
+           where id = ? and invnumber is null|;
+    $self->{dbh}->do($query,
+                     {},
+                     $self->{id})
         or $self->dberror($query);
-    $sth->execute($self->{id}) or $self->dberror($query);
-    my ($invnumber) = $sth->fetchrow_array;
-    $self->dberror($query) if $sth->err;
-
-    return if defined $invnumber or !$sth->rows;
-    $sth->finish;
-    $query = 'update ar set invnumber = ? where id = ?';
-    $sth = $self->{dbh}->prepare($query)
-        or $self->dberror($query);
-    $sth->execute(
-        $self->update_defaults(
-            $LedgerSMB::App_State::User,
-            'sinumber'
-        ),
-        $self->{id}
-    ) or $self->dberror($query);
 }
 
 =item $form->db_prepare_vars(var1, var2, ..., varI<n>)
@@ -3262,7 +3058,7 @@ sub get_batch_description {
 
 }
 
-=item sequence_dropdown(setting_key)
+=item sequence_dropdown(setting_key, readonly)
 
 This function returns the HTML code for a dropdown box for a given setting
 key.  It is not generally to be used with code on new templates.
@@ -3270,10 +3066,11 @@ key.  It is not generally to be used with code on new templates.
 =cut
 
 sub sequence_dropdown{
-    my ($self, $setting_key) = @_;
+    my ($self, $setting_key, $readonly) = @_;
     return undef if $self->{id} and ($setting_key ne 'sinumber');
     my @sequences = LedgerSMB::Setting::Sequence->list($setting_key);
-    my $retval = qq|<select data-dojo-type="dijit/form/Select" name='setting_sequence' class='sequence'>\n|;
+    $readonly = $readonly ? 'readonly="readonly"' : '';
+    my $retval = qq|<select data-dojo-type="dijit/form/Select" name='setting_sequence' class='sequence' $readonly>\n|;
     $retval .= qq|<option></option>|;
 
     for my $seq (@sequences){
@@ -3289,7 +3086,7 @@ sub sequence_dropdown{
         return $retval;
     }
     else {
-        return undef
+        return '';
     }
 }
 #end decrysiption
