@@ -11,18 +11,24 @@ CREATE TYPE draft_search_result AS (
         transdate date,
         invoice bool,
         reference text,
+        eca_name text,
         description text,
         type text,
         amount numeric
 );
 
-CREATE OR REPLACE FUNCTION draft__search(in_type text, in_with_accno text,
+DROP FUNCTION IF EXISTS draft__search(in_type text, in_with_accno text,
+in_from_date date, in_to_date date, in_amount_lt numeric, in_amount_gt numeric);
+
+CREATE OR REPLACE FUNCTION draft__search(in_type text, in_reference text,
 in_from_date date, in_to_date date, in_amount_lt numeric, in_amount_gt numeric)
 returns setof draft_search_result AS
 $$
-        SELECT id, transdate, invoice, reference, description,
+BEGIN
+RETURN QUERY EXECUTE $sql$
+        SELECT id, transdate, invoice, reference, eca_name, description,
                type, amount FROM (
-            SELECT id, transdate, reference,
+            SELECT id, transdate, reference, null::text as eca_name,
                    description, false as invoice,
                    (SELECT SUM(line.amount_bc)
                       FROM acc_trans line
@@ -30,46 +36,41 @@ $$
                            and line.trans_id = gl.id) as amount,
                    'gl' as type
               from gl
-             WHERE (lower(in_type) = 'gl' or in_type is null)
+             WHERE (lower($1) = 'gl' or $1 is null)
                   AND NOT approved
                   AND NOT EXISTS (SELECT 1
                                     FROM voucher v
                                    WHERE v.trans_id = gl.id)
             UNION
             SELECT id, transdate, invnumber as reference,
-                (SELECT name FROM eca__get_entity(entity_credit_account)),
-                invoice, amount_bc as amount, 'ap' as type
+                (SELECT name FROM eca__get_entity(entity_credit_account)) as eca_name,
+                description, invoice, amount_bc as amount, 'ap' as type
               FROM ap
-             WHERE (lower(in_type) = 'ap' or in_type is null)
+             WHERE (lower($1) = 'ap' or $1 is null)
                    AND NOT approved
                    AND NOT EXISTS (SELECT 1
                                      FROM voucher v
                                     WHERE v.trans_id = ap.id)
             UNION
             SELECT id, transdate, invnumber as reference,
+                (SELECT name FROM eca__get_entity(entity_credit_account)) as eca_name,
                 description, invoice, amount_bc as amount, 'ar' as type
               FROM ar
-             WHERE (lower(in_type) = 'ar' or in_type is null)
+             WHERE (lower($1) = 'ar' or $1 is null)
                    AND NOT approved
                    AND NOT EXISTS (SELECT 1
                                      FROM voucher v
                                     WHERE v.trans_id = ar.id)) trans
-        WHERE (in_from_date IS NULL or trans.transdate >= in_from_date)
-          AND (in_to_date IS NULL or trans.transdate <= in_to_date)
-          AND (in_amount_gt IS NULL or amount >= in_amount_gt)
-          AND (in_amount_lt IS NULL or amount <= in_amount_lt)
-          AND (in_with_accno IS NULL
-               OR id IN (SELECT line.trans_id
-                           FROM acc_trans line
-                           JOIN account acc ON (line.chart_id = acc.id)
-                          WHERE acc.accno = in_with_accno
-                            AND NOT approved
-                            AND (in_from_date IS NULL
-                                 OR line.transdate >= in_from_date)
-                            AND (in_to_date IS NULL
-                                 OR line.transdate <= in_to_date)))
-        ORDER BY trans.reference;
-$$ language sql;
+        WHERE ($3 IS NULL or trans.transdate >= $3)
+          AND ($4 IS NULL or trans.transdate <= $4)
+          AND ($6 IS NULL or amount >= $6)
+          AND ($5 IS NULL or amount <= $5)
+          AND ($2 IS NULL or trans.reference = $2)
+        ORDER BY trans.reference
+$sql$
+USING in_type, in_reference, in_from_date, in_to_date, in_amount_lt, in_amount_gt;
+END
+$$ LANGUAGE PLPGSQL;
 
 COMMENT ON FUNCTION draft__search(in_type text, in_with_accno text,
 in_from_date date, in_to_date date, in_amount_le numeric, in_amount_ge numeric)
@@ -107,6 +108,10 @@ begin
                 approved_at = now()
         WHERE id = in_id;
 
+        UPDATE acc_trans
+        SET approved = 't'::boolean
+        WHERE trans_id = in_id;
+
         RETURN TRUE;
 END;
 $$ LANGUAGE PLPGSQL SECURITY DEFINER;
@@ -119,12 +124,28 @@ $$
 declare
         t_table text;
 begin
-        DELETE FROM ac_tax_form
-        WHERE entry_id IN
-                (SELECT entry_id FROM acc_trans WHERE trans_id = in_id);
+        DELETE FROM ac_tax_form atf
+         WHERE EXISTS (SELECT 1 FROM acc_trans
+                        WHERE entry_id = atf.entry_id
+                              AND trans_id = in_id);
+
+        DELETE FROM payment_links pl
+         WHERE EXISTS (SELECT 1 FROM acc_trans
+                        WHERE entry_id = pl.entry_id
+                              AND trans_id = in_id)
+               AND (SELECT count(distinct ac.trans_id)
+                      FROM payment p
+                      JOIN payment_links pli ON p.id = pli.payment_id
+                      JOIN acc_trans ac ON pli.entry_id = ac.entry_id
+                     WHERE pl.payment_id = p.id) <= 1;
 
         DELETE FROM acc_trans WHERE trans_id = in_id;
-        SELECT lower(table_name) into t_table FROM transactions where id = in_id;
+        DELETE FROM invoice_tax_form itf
+           WHERE EXISTS (select 1 from invoice i
+                          where i.trans_id = in_id and itf.invoice_id = i.id);
+        DELETE FROM invoice WHERE trans_id = in_id;
+        SELECT lower(table_name) into t_table
+          FROM transactions where id = in_id;
 
         IF t_table = 'ar' THEN
                 DELETE FROM ar WHERE id = in_id AND approved IS FALSE;
