@@ -1,45 +1,154 @@
 
 set client_min_messages = 'warning';
 
-
--- The reconciliation reports have the following state transition diagram:
+/*
+The reconciliation reports have the following state transition diagram:
 
 
--- +----------+    +--------+    +------------+    +------------+
--- | Initial  +--->+ Saved  +--->+ Submitted  +-+->+ Accepted   |
--- +----------+    +-+------+    +------+-----+ |  +------------+
---                   | ^                |       |
---                   | \---Rejecting----/       |  +------------+
---                   \--------------------------+->+ Deleted    |
---                                                 -------------+
++----------+    +--------+    +------------+     +------------+
+| Initial  +--->+ Saved  +--->+ Submitted  +-+-->+ Approved   |
++----------+    +-+------+    +------+-----+ |   +------------+
+                  | ^                |       |
+                  | \---Rejecting----/       |   +------------+
+                  \-------------------------/ \->+ Deleted    |
+                                                 +------------+
 
--- lines from acc_trans are referenced in the report lines. The cr_report_lines
--- are marked 'cleared' as soon as they are marked reconciled (and saved) in
--- the reconciliation screen.
+The state diagram is reflected in the various table columns as follows
+(excluding the 'deleted' state, which has no rows in the database):
 
--- When a report is Rejected, it's returned to the saved state for correction.
+|------------------------------+---------+-------+-----------+----------|
+| table column                 | initial | saved | submitted | approved |
+|------------------------------+---------+-------+-----------+----------|
+| cr_report.submitted          | false   | false | true      | true     |
+| cr_report.approved           | false   | false | false     | true     |
+| cr_report_line.cleared       | false   | t/f   | t/f       | t/f      |
+| cr_report_line_links.cleared | false   | false | true (2)  | true (2) |
+| acc_trans.cleared            | false   | false | false     | true (1) |
+|------------------------------+---------+-------+-----------+----------|
 
--- Upon *approval*, the 'cleared' status is written to the 'acc_trans' table,
--- which means that rejected or deleted reports don't have any impact on
--- the reconciliation state of the actual transactions.
+(1): Only for those lines which have a corresponding cr_report_line_link
+record which is also marked "cleared". Note that a reconciliation report
+can have both cleared and uncleared lines in all states.
+
+(2): For all rows where the cr_report_line.cleared column is marked "cleared".
 
 
+The reasoning behind the difference in definition of the "cleared" column
+between the cr_report_line, cr_report_line_link and acc_trans tables is as
+follows:
+
+ * cr_report_line.cleared indicates if the line on the report is considered
+   to have cleared
+ * cr_report_line_link.cleared indicates (the intention to mark) an acc_trans
+   line as being cleared; the 'submitted' state comes with this intention
+ * acc_trans.cleared indicates that the line actually has been cleared
+
+
+|------------+-----------+--------------------------------------|
+| From state | To state  | Function name                        |
+|------------+-----------+--------------------------------------|
+| <start>    | Initial   | reconciliation__new_repord_id        |
+| Initial    | Saved     | reconciliation__save_set             |
+| Saved      | Submitted | reconciliation__submit_set           |
+| Submitted  | Approved  | reconciliation__report_approve       |
+| Submitted  | Saved     | reconciliation__reject_set           |
+| Iniitial   | Deleted   | reconciliation__delete_unapproved    |
+| Saved      | Deleted   | reconciliation__delete_unapproved    |
+| Submitted  | Deleted   | reconciliation__delete_unapproved    |
+| Initial    | Deleted   | reconciliation__delete_my_report     |
+| Saved      | Deleted   | reconciliation__delete_my_report     |
+| Initial    | Initial   | recenciliation__pending_transactions |
+| Saved      | Saved     | recenciliation__pending_transactions |
+| Initial    | Initial   | reconciliation__add_entry            |
+| Saved      | Saved     | reconciliation__add_entry            |
+|------------+-----------+--------------------------------------|
+
+
+
+
+
+*/
 
 BEGIN;
 
-CREATE OR REPLACE FUNCTION reconciliation__submit_set(
-        in_report_id int, in_line_ids int[]) RETURNS bool AS
+CREATE OR REPLACE FUNCTION cr_report_submitted_update()
+RETURNS trigger
+AS
+$$
+BEGIN
+  UPDATE cr_report_line_links rll
+     SET cleared = rl.cleared and NEW.submitted
+    FROM cr_report_line rl
+   WHERE rll.report_line_id = rl.id
+         AND rl.report_id = NEW.id;
+
+  RETURN NEW;
+END;
+$$ LANGUAGE PLPGSQL;
+
+CREATE OR REPLACE FUNCTION cr_report_line_cleared_update()
+RETURNS trigger
+AS
+$$
+BEGIN
+  UPDATE cr_report_line_links rll
+     SET cleared = NEW.cleared and r.submitted
+    FROM cr_report r
+   WHERE rll.report_line_id = NEW.id
+         AND r.id = NEW.report_id;
+
+  RETURN NEW;
+END;
+$$ LANGUAGE PLPGSQL;
+
+CREATE OR REPLACE FUNCTION cr_report_line_link_insert()
+RETURNS trigger
+AS
+$$
+BEGIN
+  NEW.cleared = (select r.submitted and rl.cleared
+                   from cr_report_line rl
+                   join cr_report r on rl.report_id = r.id
+                  where rl.id = NEW.report_line_id);
+
+  RETURN NEW;
+END;
+$$ LANGUAGE PLPGSQL;
+
+-- drop and create, because only Pg14+ support CREATE OR REPLACE
+DROP TRIGGER IF EXISTS cr_report_links_update ON cr_report;
+CREATE TRIGGER cr_report_links_update AFTER UPDATE OF submitted
+    ON cr_report
+    FOR EACH ROW
+    EXECUTE PROCEDURE cr_report_submitted_update();
+
+-- drop and create, because only Pg14+ support CREATE OR REPLACE
+DROP TRIGGER IF EXISTS cr_report_line_links_update ON cr_report_line;
+CREATE TRIGGER cr_report_line_links_update AFTER UPDATE OF cleared
+    ON cr_report_line
+    FOR EACH ROW
+    EXECUTE PROCEDURE cr_report_line_cleared_update();
+
+-- drop and create, because only Pg14+ support CREATE OR REPLACE
+DROP TRIGGER IF EXISTS cr_report_line_link_insert ON cr_report_line_links;
+CREATE TRIGGER cr_report_line_link_insert BEFORE INSERT
+    ON cr_report_line_links
+    FOR EACH ROW
+    EXECUTE PROCEDURE cr_report_line_link_insert();
+
+-- drop and recreate because of the signature change.
+DROP FUNCTION IF exists reconciliation__submit_set(in_report_id integer, in_line_ids integer[]);
+CREATE OR REPLACE FUNCTION reconciliation__submit_set(in_report_id int)
+RETURNS bool AS
 $$
 BEGIN
         UPDATE cr_report set submitted = true where id = in_report_id;
-        PERFORM reconciliation__save_set(in_report_id, in_line_ids);
 
         RETURN FOUND;
 END;
 $$ LANGUAGE PLPGSQL;
 
-COMMENT ON FUNCTION reconciliation__submit_set(
-        in_report_id int, in_line_ids int[]) IS
+COMMENT ON FUNCTION reconciliation__submit_set(in_report_id int) IS
 $$Submits a reconciliation report for approval.
 in_line_ids is used to specify which report lines are cleared, finalizing the
 report.$$;
@@ -96,12 +205,10 @@ to reject approval.$$;
 CREATE OR REPLACE FUNCTION reconciliation__save_set(
         in_report_id int, in_line_ids int[]) RETURNS bool AS
 $$
-        UPDATE cr_report_line SET cleared = false
-        WHERE report_id = in_report_id;
+        UPDATE cr_report_line SET cleared = (id = ANY(in_line_ids))
+         WHERE report_id = in_report_id;
 
-        UPDATE cr_report_line SET cleared = true
-        WHERE report_id = in_report_id AND id = ANY(in_line_ids)
-        RETURNING TRUE;
+        SELECT TRUE;
 $$ LANGUAGE SQL;
 
 COMMENT ON FUNCTION reconciliation__save_set(
@@ -123,11 +230,6 @@ $$
     RETURNING TRUE;
 $$ LANGUAGE SQL SECURITY DEFINER;
 
--- Granting execute permission to public because everyone has an ability to
--- delete their own reconciliation reports provided they have not been
--- submitted.  --CT
-GRANT EXECUTE ON FUNCTION reconciliation__delete_my_report(in_report_id int)
-TO PUBLIC;
 
 COMMENT ON FUNCTION reconciliation__delete_my_report(in_report_id int) IS
 $$This function allows a user to delete his or her own unsubmitted, unapproved
@@ -186,33 +288,46 @@ CREATE OR REPLACE FUNCTION reconciliation__get_cleared_balance(in_chart_id int,
    in_report_date date DEFAULT date_trunc('second', now()))
 RETURNS numeric AS
 $$
-    SELECT sum(ac.amount_bc) * CASE WHEN c.category in('A', 'E') THEN -1 ELSE 1 END
-        FROM account c
-        JOIN acc_trans ac ON (ac.chart_id = c.id)
-    JOIN (select id from transactions where approved) g ON g.id = ac.trans_id
-    WHERE c.id = $1 AND cleared
-      AND ac.approved IS true
-      AND ac.transdate <= in_report_date
+  SELECT sum(ac.amount_bc) * CASE WHEN c.category in('A', 'E') THEN -1 ELSE 1 END
+    FROM account c
+           JOIN acc_trans ac ON ac.chart_id = c.id
+           JOIN transactions g ON g.id = ac.trans_id
+   WHERE g.approved
+     AND c.id = in_chart_id
+     AND ac.approved
+     -- cleared using a report on or before in_report_date:
+     AND EXISTS (select 1
+                   from cr_report cr
+                          join cr_report_line crl on cr.id = crl.report_id
+                          join cr_report_line_links crll on crl.id = crll.report_line_id
+                  where cr.approved
+                    and cr.chart_id = in_chart_id
+                    and cr.end_date <= in_report_date
+                    and crl.cleared
+                    and crll.entry_id = ac.entry_id)
     GROUP BY c.id, c.category;
 $$ LANGUAGE sql;
 
 COMMENT ON FUNCTION reconciliation__get_cleared_balance(in_chart_id int,in_report_date date) IS
-$$ Gets the cleared balance of the account specified by chart_id, as of in_report_date.
-This is specified in normal format (i.e. positive numbers for debits for asset
-and espense accounts, and positive numbers for credits in other accounts
+$$ Gets the cleared balance of the account specified by chart_id, as cleared by reports
+on and before in_report_date.
+
+Please note that the cleared balance amount as at a sperific date may differ from the value
+returned by this function, if transactions prior to in_report_date are cleared using reports
+on a date later than in_report_date.
+
+The returned value is specified in normal format (i.e. positive numbers for debits for asset
+and expense accounts, and positive numbers for credits in other accounts.
 
 Note that currently contra accounts will show negative balances.$$;
 
 CREATE OR REPLACE FUNCTION reconciliation__report_approve (in_report_id INT) returns INT as $$
 
-    DECLARE
-        current_row RECORD;
-        ac_entries int[];
     BEGIN
         UPDATE cr_report SET approved = 't',
                 approved_by = person__get_my_entity_id(),
                 approved_username = SESSION_USER
-        WHERE id = in_report_id;
+         WHERE id = in_report_id;
         IF NOT FOUND THEN
             RAISE EXCEPTION 'No report at %.', $1;
         END IF;
@@ -225,7 +340,6 @@ CREATE OR REPLACE FUNCTION reconciliation__report_approve (in_report_id INT) ret
                         where rll.entry_id = ac.entry_id
                               and rl.cleared
                               and rl.report_id = in_report_id);
-
         return 1;
     END;
 
@@ -236,11 +350,15 @@ $$Marks the report approved and marks all cleared transactions in it cleared.$$;
 
 
 -- XXX Badly named, rename for 1.4.  --CT
-CREATE OR REPLACE FUNCTION reconciliation__new_report_id
-(in_chart_id int, in_total numeric, in_end_date date, in_recon_fx bool) returns INT as $$
+CREATE OR REPLACE FUNCTION reconciliation__new_report_id(
+    in_chart_id int,
+    in_total numeric,
+    in_end_date date,
+    in_recon_fx bool
+) returns INT as $$
 
     INSERT INTO cr_report(chart_id, their_total, end_date, recon_fx)
-    values ($1, $2, $3, $4);
+    values (in_chart_id, in_total, in_end_date, in_recon_fx);
     SELECT currval('cr_report_id_seq')::int;
 
 $$ language 'sql';
@@ -261,7 +379,6 @@ CREATE OR REPLACE FUNCTION reconciliation__add_entry(
         in_account int;
         la RECORD;
         t_errorcode INT;
-        our_value NUMERIC;
         lid INT;
         in_count int;
         t_scn TEXT;
@@ -304,7 +421,7 @@ CREATE OR REPLACE FUNCTION reconciliation__add_entry(
                                 in_type)
                         RETURNING id INTO lid;
                 ELSIF in_count = 1 THEN
-                        SELECT id INTO lid
+                        SELECT id INTO lid FROM cr_report_line
                         WHERE t_scn = scn AND report_id = in_report_id
                                 AND their_balance = 0 AND post_date = in_date;
                         UPDATE cr_report_line
@@ -314,7 +431,7 @@ CREATE OR REPLACE FUNCTION reconciliation__add_entry(
                 ELSE
                         SELECT count(*) INTO in_count FROM cr_report_line
                         WHERE t_scn ilike scn AND report_id = in_report_id
-                                AND our_value = t_amount and their_balance = 0
+                                AND our_balance = t_amount and their_balance = 0
                                 AND post_date = in_date;
 
                         IF in_count = 0 THEN -- no match among many of values
@@ -334,7 +451,7 @@ CREATE OR REPLACE FUNCTION reconciliation__add_entry(
                         ELSIF in_count = 1 THEN -- EXECT MATCH
                                 SELECT id INTO lid FROM cr_report_line
                                 WHERE t_scn = scn AND report_id = in_report_id
-                                        AND our_value = t_amount
+                                        AND our_balance = t_amount
                                         AND their_balance = 0
                                         AND post_date = in_date;
                                 UPDATE cr_report_line
@@ -346,7 +463,7 @@ CREATE OR REPLACE FUNCTION reconciliation__add_entry(
                         ELSE -- More than one match
                                 SELECT id INTO lid FROM cr_report_line
                                 WHERE t_scn ilike scn AND report_id = in_report_id
-                                        AND our_value = t_amount
+                                        AND our_balance = t_amount
                                         AND post_date = in_date
                                 ORDER BY id ASC limit 1;
 
@@ -374,7 +491,7 @@ CREATE OR REPLACE FUNCTION reconciliation__add_entry(
                         in_type)
                         RETURNING id INTO lid;
                 ELSIF in_count = 1 THEN -- perfect match
-                        SELECT id INTO lid
+                        SELECT id INTO lid FROM cr_report_line
                         WHERE report_id = in_report_id
                                 AND our_balance = t_amount
                                 AND their_balance = 0
@@ -428,8 +545,10 @@ DROP FUNCTION IF EXISTS
                                        in_report_id integer,
                                        in_their_total numeric);
 CREATE OR REPLACE FUNCTION reconciliation__pending_transactions(
-                      in_report_id integer, in_their_total numeric)
-  RETURNS integer AS
+    in_report_id integer,
+    in_their_total numeric
+)
+RETURNS integer AS
 $$
 
     DECLARE
@@ -438,23 +557,52 @@ $$
         t_chart_id       integer;
         t_end_date       date;
         t_report_line_id integer;
+        t_uid int;
     BEGIN
         SELECT end_date, recon_fx, chart_id
          INTO t_end_date, t_recon_fx, t_chart_id
          FROM cr_report
         WHERE id = in_report_id;
 
+        SELECT entity_id INTO t_uid
+        FROM users
+        WHERE username = CURRENT_USER;
+
         /*
 
-        Approach:
+        Approach in 4 steps:
          1. Identify lines to be added *somewhere*
+            That is: all lines before the reconcilation date which
+            are not yet part of any other reconciliation; lines come
+            from two sources: payment transactions and others (the second
+            are usually GL transactions)
          2. Identify lines part of a payment
-            (always added as new lines: payments are natural groupings)
-         3. Identify non-payment lines to be added to existing
-            recon lines due to the same source system reference
-         4. Remaining lines added as new lines, either by source or
-            as individual ones.
-
+            Lines in this category are grouped by payment and added as a
+            single reconciliation line, irrespective of the number of lines
+            identified, *unless* lines have explicitly different 'Source'
+            values - which is weird and unexpected, but possible when the
+            user sets a specific value on each payment line separately - in
+            which case, the lines in the payment will be grouped by the value
+            of the Source field
+         3. Identify non-payment lines that adjust payments
+            When a payment has been entered wrongly or the bank has withheld
+            transaction fees, the payment of the invoice does not correspond
+            to the actual amount on the bank statement - meaning adjustment
+            is required; GL transactions can be used to enter adjustments by
+            listing the same date and the same source as used for the payment
+            transaction. The lines in this category will be added as an
+            adjustment to the existing (coming from the payment) reconciliation
+            line
+         4. Remaining lines added as new lines, either by source (if they
+            have one) or as individual ones.
+            Note that the lines in this category - by logical reasoning - can
+            **not** be payments lines, because those were handled in step 2.
+            Also note that it's not an option to lump all lines without a source
+            into a single line, because that way all lines without a Source
+            would end up as a single reconciliation line, while unknowing users
+            are expected to post GL lines without Source numbers; to help these
+            users, we present lines from non-payment (GL) transactions as
+            individual lines
          */
 
         -- step 1: identify lines to be added somehow
@@ -478,19 +626,17 @@ $$
            select payment_id, array_agg(ac.entry_id) as entries,
                   sum(case when t_recon_fx then amount_tc
                            else amount_bc end) as our_balance,
-                  payment_date
+                  payment_date, source
              from payment_links pl
              join acc_trans ac on pl.entry_id = ac.entry_id
              join payment p on p.id = pl.payment_id
             where ac.chart_id = t_chart_id
                   and pl.entry_id in (select entry_id from lines_to_be_added)
-           group by payment_id, payment_date
+           group by payment_id, payment_date, source
         loop
-           insert into cr_report_line (report_id, their_balance,
+            insert into cr_report_line (report_id, scn, their_balance,
                                        our_balance, post_date, "user")
-              values (in_report_id, 0, t_row.our_balance, t_row.payment_date,
-                      (select entity_id from users
-                        where username = CURRENT_USER))
+            values (in_report_id, t_row.source, 0, t_row.our_balance, t_row.payment_date, t_uid)
            returning id into t_report_line_id;
 
            update lines_to_be_added
@@ -507,22 +653,49 @@ $$
                         join acc_trans ac on ac.source = rl.scn
                                              and ac.transdate = rl.post_date
                        where la.entry_id = ac.entry_id
-                             and rl.report_id = in_report_id)
+                             and rl.report_id = in_report_id
+                             -- exclude 'scn' values associated with more than one
+                             -- report line: the gl line can't be unambiguously
+                             -- combined with a payment; hence it can't serve as
+                             -- a correction...
+                             and not exists (select 1 from cr_report_line rli
+                                              where rl.post_date = rli.post_date
+                                                    and rl.report_id = rli.report_id
+                                                    and rli.scn = rl.scn
+                                                    and rl.id <> rli.id)
+                             and not exists (select 1 from payment_links pl
+                                              where pl.entry_id = ac.entry_id))
             where la.report_line_id is null
-           returning report_line_id
+           returning report_line_id, entry_id
         )
         update cr_report_line rl
            set our_balance = (select sum(case when t_recon_fx then ac.amount_tc
                                               else ac.amount_bc end)
-                                from cr_report_line_links rll
+                                from (
+                                     -- lines that were already there
+                                     select report_line_id, entry_id
+                                       from cr_report_line_links
+                                     union all
+                                     -- lines identified in step (2)
+                                     -- (does not include 'matched_entries', because
+                                     -- the default transaction isolation [read
+                                     -- committed] freezes our view at query start,
+                                     -- which means lines_to_be_added isn't updated
+                                     -- by 'matched_entries', as we see it)
+                                     select report_line_id, entry_id
+                                       from lines_to_be_added
+                                     union all
+                                     -- lines identidief in this step
+                                     select report_line_id, entry_id
+                                       from matched_entries
+                                ) rll
                                 join acc_trans ac on rll.entry_id = ac.entry_id
                                 where rl.id = rll.report_line_id)
          where rl.id in (select report_line_id from matched_entries);
 
         -- step 4: add new lines not part of payments
         for t_row in
-           select source, case when source is null then entry_id
-                               else null end, array_agg(entry_id) as entries,
+           select source, array_agg(entry_id) as entries,
                   sum(case when t_recon_fx then amount_tc
                            else amount_bc end) as our_balance,
                   transdate
@@ -535,10 +708,8 @@ $$
         loop
            insert into cr_report_line (report_id, scn, their_balance,
                                       our_balance, post_date, "user")
-              values (in_report_id, t_row.source, 0,
-                      t_row.our_balance, t_row.transdate,
-                      (select entity_id from users
-                        where username = CURRENT_USER))
+            values (in_report_id, t_row.source, 0,
+                      t_row.our_balance, t_row.transdate, t_uid)
            returning id into t_report_line_id;
 
            update lines_to_be_added
@@ -679,9 +850,13 @@ account.  For asset and expense accounts this is the debit balance, for others
 this is the credit balance.$$;
 
 CREATE OR REPLACE VIEW recon_payee AS
- SELECT n.name AS payee, rr.id, rr.report_id, rr.scn, rr.their_balance, rr.our_balance, rr.errorcode, rr."user", rr.clear_time, rr.insert_time, rr.trans_type, rr.post_date, rr.ledger_id, ac.voucher_id, rr.overlook, rr.cleared
+ SELECT DISTINCT ON (rr.id)
+   n.name AS payee, rr.id, rr.report_id, rr.scn, rr.their_balance,
+   rr.our_balance, rr."user", rr.clear_time, rr.insert_time, rr.trans_type,
+   rr.post_date, rr.cleared
    FROM cr_report_line rr
-   LEFT JOIN acc_trans ac ON rr.ledger_id = ac.entry_id
+   LEFT JOIN cr_report_line_links rll ON rr.id = rll.report_line_id
+   LEFT JOIN acc_trans ac ON rll.entry_id = ac.entry_id
    LEFT JOIN gl ON ac.trans_id = gl.id
    LEFT JOIN (( SELECT ap.id, e.name
    FROM ap
