@@ -51,7 +51,8 @@ CREATE TYPE voucher_list AS (
 CREATE OR REPLACE FUNCTION voucher__list (in_batch_id integer)
 RETURNS SETOF voucher_list AS
 $$
-                SELECT v.id, a.invoice, a.invnumber, e.name,
+                SELECT v.id, a.invoice, a.invnumber,
+                        eca.meta_number || '--' || e.name,
                         v.batch_id, v.trans_id,
                         a.amount_bc, a.transdate, 'Payable', v.batch_class
                 FROM voucher v
@@ -63,7 +64,8 @@ $$
                         AND v.batch_class = (select id from batch_class
                                         WHERE class = 'ap')
                 UNION
-                SELECT v.id, a.invoice, a.invnumber, e.name,
+                SELECT v.id, a.invoice, a.invnumber,
+                        eca.meta_number || '--' || e.name,
                         v.batch_id, v.trans_id,
                         a.amount_bc, a.transdate, 'Receivable', v.batch_class
                 FROM voucher v
@@ -76,8 +78,8 @@ $$
                                         WHERE class = 'ar')
                 UNION ALL
                 -- TODO:  Add the class labels to the class table.
-                SELECT v.id, false, a.source,
-                        cr.meta_number || '--'  || co.legal_name ,
+                SELECT v.id, ap.invoice, a.source,
+                        eca.meta_number || '--'  || e.name,
                         v.batch_id, v.trans_id,
                         sum(CASE WHEN bc.class LIKE 'payment%' THEN a.amount_bc * -1
                              ELSE a.amount_bc  END), a.transdate,
@@ -90,17 +92,18 @@ $$
                 JOIN batch_class bc ON (bc.id = v.batch_class)
                 JOIN account_link l ON (a.chart_id = l.account_id)
                 JOIN ap ON (ap.id = a.trans_id)
-                JOIN entity_credit_account cr
-                        ON (ap.entity_credit_account = cr.id)
-                JOIN company co ON (cr.entity_id = co.entity_id)
+                JOIN entity_credit_account eca
+                        ON (ap.entity_credit_account = eca.id)
+                JOIN entity e ON (eca.entity_id = e.id)
                 WHERE v.batch_id = in_batch_id
                         AND a.voucher_id = v.id
                         AND (bc.class like 'payment%' AND l.description = 'AP')
-                GROUP BY v.id, a.source, cr.meta_number, co.legal_name ,
+                GROUP BY v.id, ap.invoice, a.source, eca.meta_number, e.name,
                         v.batch_id, v.trans_id, a.transdate, bc.class
 
                 UNION ALL
-                SELECT v.id, false, a.source, a.memo,
+                SELECT v.id, ar.invoice, a.source,
+                        eca.meta_number || '--'  || e.name,
                         v.batch_id, v.trans_id,
                         CASE WHEN bc.class LIKE 'receipt%' THEN sum(a.amount_bc) * -1
                              ELSE sum(a.amount_bc)  END, a.transdate,
@@ -113,14 +116,14 @@ $$
                 JOIN batch_class bc ON (bc.id = v.batch_class)
                 JOIN account_link l ON (a.chart_id = l.account_id)
                 JOIN ar ON (ar.id = a.trans_id)
-                JOIN entity_credit_account cr
-                        ON (ar.entity_credit_account = cr.id)
-                JOIN company co ON (cr.entity_id = co.entity_id)
+                JOIN entity_credit_account eca
+                        ON (ar.entity_credit_account = eca.id)
+                JOIN entity e ON (eca.entity_id = e.id)
                 WHERE v.batch_id = in_batch_id
                         AND a.voucher_id = v.id
                         AND (bc.class like 'receipt%' AND l.description = 'AR')
-                GROUP BY v.id, a.source, cr.meta_number, co.legal_name ,
-                        a.memo, v.batch_id, v.trans_id, a.transdate, bc.class
+                GROUP BY v.id, ar.invoice, a.source, eca.meta_number, e.name,
+                        v.batch_id, v.trans_id, a.transdate, bc.class
                 UNION ALL
                 SELECT v.id, false, g.reference, g.description,
                         v.batch_id, v.trans_id,
@@ -365,6 +368,13 @@ $$
         WHERE id IN (select trans_id FROM voucher
                 WHERE batch_id = in_batch_id);
 
+        -- When approving the AR/AP batch import,
+        -- we need to approve the acc_trans line also.
+        UPDATE acc_trans SET approved = true
+        WHERE trans_id IN (select trans_id FROM voucher
+                WHERE batch_id = in_batch_id
+                AND batch_class IN (1, 2));
+
         UPDATE acc_trans SET approved = true
         WHERE voucher_id IN (select id FROM voucher
                 WHERE batch_id = in_batch_id
@@ -448,17 +458,17 @@ BEGIN
                        (select id from voucher where batch_id = in_batch_id)
                );
 
-        SELECT as_array(payment_id) INTO t_payment_ids
-          FROM payment_links
-         WHERE EXISTS (select 1 from payment_links p join acc_trans a
-                           on p.entry_id = a.entry_id
-                        where a.voucher_id IN (select id from voucher
-                                                where batch_id = in_batch_id));
-        DELETE FROM payment_links
-         WHERE EXISTS (select 1 from payment_links p join acc_trans a
-                           on p.entry_id = a.entry_id
-                        where a.voucher_id IN (select id from voucher
-                                                where batch_id = in_batch_id));
+        WITH deleted_payment_ids AS (
+           DELETE FROM payment_links p
+            WHERE EXISTS (select 1 from acc_trans a
+                           where p.entry_id = a.entry_id
+                                 and a.voucher_id IN (select id from voucher
+                                                       where batch_id = in_batch_id))
+         RETURNING p.payment_id
+        )
+        SELECT array_agg(payment_id) INTO t_payment_ids
+          FROM deleted_payment_ids;
+
         DELETE FROM payment
          WHERE id = any(t_payment_ids)
                AND NOT EXISTS (select 1 from payment_links
@@ -470,7 +480,7 @@ BEGIN
         -- The rest of this function involves the deletion of actual
         -- transactions, vouchers, and batches, and jobs which are in progress.
         -- -CT
-        SELECT as_array(trans_id) INTO t_transaction_ids
+        SELECT array_agg(trans_id) INTO t_transaction_ids
         FROM voucher WHERE batch_id = in_batch_id AND batch_class IN (1, 2, 5, 8, 9);
 
         DELETE FROM ac_tax_form WHERE entry_id in
@@ -482,11 +492,11 @@ BEGIN
 
         DELETE FROM invoice WHERE trans_id = ANY(t_transaction_ids);
         DELETE FROM acc_trans WHERE trans_id = ANY(t_transaction_ids);
+        DELETE FROM voucher WHERE batch_id = in_batch_id;
+        DELETE FROM batch WHERE id = in_batch_id;
         DELETE FROM ar WHERE id = ANY(t_transaction_ids);
         DELETE FROM ap WHERE id = ANY(t_transaction_ids);
         DELETE FROM gl WHERE id = ANY(t_transaction_ids);
-        DELETE FROM voucher WHERE batch_id = in_batch_id;
-        DELETE FROM batch WHERE id = in_batch_id;
         DELETE FROM transactions WHERE id = ANY(t_transaction_ids);
 
         RETURN 1;
@@ -530,6 +540,9 @@ BEGIN
                                  where pl.payment_id = p.id);
         DELETE FROM acc_trans WHERE trans_id = voucher_row.trans_id;
 
+        -- deletion of the ar/ap/gl row causes removal of the `transactions`
+        -- row, which fails if the voucher isn't deleted...
+        DELETE FROM voucher WHERE id = voucher_row.id;
         DELETE FROM ar WHERE id = voucher_row.trans_id;
         DELETE FROM ap WHERE id = voucher_row.trans_id;
         DELETE FROM gl WHERE id = voucher_row.trans_id;
@@ -552,9 +565,9 @@ BEGIN
                 AND NOT EXISTS (select 1 from payment_links pl
                                  where pl.payment_id = p.id);
         DELETE FROM acc_trans where voucher_id = voucher_row.id;
+        DELETE FROM voucher WHERE id = voucher_row.id;
     END IF;
 
-    DELETE FROM voucher WHERE id = voucher_row.id;
     RETURN 1;
 END;
 $$ LANGUAGE PLPGSQL SECURITY DEFINER;
