@@ -5,6 +5,7 @@
 -- to all Ledgersmb up to 1.6
 
 -- When moved to an interface, these will all be specified and preprocessed.
+set client_min_messages to 'ERROR';
 \set default_country '''[% default_country %]'''
 \set ar '''[% default_ar %]'''
 \set ap '''[% default_ap %]'''
@@ -17,9 +18,6 @@
 
 BEGIN;
 
-ALTER TABLE :slschema.acc_trans DROP COLUMN IF EXISTS lsmb_entry_id;
-ALTER TABLE :slschema.acc_trans add column lsmb_entry_id SERIAL UNIQUE;
-
 -- Migration functions
 -- TODO: Can we do without?
 
@@ -56,7 +54,7 @@ BEGIN
             t_heading_id := in_heading;
     END IF;
 
--- don't remove custom links.
+    -- don't remove custom links.
     DELETE FROM account_link
     WHERE account_id = in_id
           and description in ( select description
@@ -256,7 +254,7 @@ CREATE OR REPLACE FUNCTION reconciliation__add_entry(
                                 in_type)
                         RETURNING id INTO lid;
                 ELSIF in_count = 1 THEN
-                        SELECT id INTO lid
+                        SELECT id INTO lid FROM cr_report_line
                         WHERE t_scn = scn AND report_id = in_report_id
                                 AND their_balance = 0 AND post_date = in_date;
                         UPDATE cr_report_line
@@ -326,7 +324,7 @@ CREATE OR REPLACE FUNCTION reconciliation__add_entry(
                         in_type)
                         RETURNING id INTO lid;
                 ELSIF in_count = 1 THEN -- perfect match
-                        SELECT id INTO lid
+                        SELECT id INTO lid FROM cr_report_line
                         WHERE report_id = in_report_id
                                 AND our_balance = t_amount
                                 AND their_balance = 0
@@ -388,7 +386,7 @@ ALTER TABLE :slschema.customer ADD COLUMN credit_id int;
 
 -- Speed optimizations
 ALTER TABLE :slschema.acc_trans DROP COLUMN IF EXISTS lsmb_entry_id;
-ALTER TABLE :slschema.acc_trans ADD COLUMN lsmb_entry_id integer;
+ALTER TABLE :slschema.acc_trans ADD COLUMN lsmb_entry_id INTEGER;
 ALTER TABLE :slschema.acc_trans ADD COLUMN type CHAR(2);
 ALTER TABLE :slschema.acc_trans ADD COLUMN accno TEXT;
 ALTER TABLE :slschema.acc_trans ADD transdate_month DATE;
@@ -411,6 +409,90 @@ UPDATE :slschema.acc_trans SET accno = (SELECT accno
                            FROM :slschema.chart
                           WHERE chart.id = :slschema.acc_trans.chart_id);
 
+
+[% IF sl_version < "2.8.3" %]
+
+-- Upgrade the <2.8.3 schema to 2.8.3 to get minimum compatibility
+
+create table :slschema.paymentmethod (
+  id int primary key default nextval('id'),
+  description text,
+  fee float
+);
+
+alter table :slschema.vendor add paymentmethod_id int;
+alter table :slschema.customer add paymentmethod_id int;
+[% END %]
+
+[% IF sl_version < "2.8.4" %]
+
+-- Upgrade the <2.8.4 schema to 2.8.4 to get minimum compatibility
+
+create sequence :slschema.paymentid;
+
+create table :slschema.payment (
+  id int not null,
+  trans_id int not null,
+  exchangerate float default 1,
+  paymentmethod_id int
+);
+
+update :slschema.acc_trans
+   set id = nextval('paymentid')
+       from :slschema.ar, :slschema.chart
+ where ar.id = acc_trans.trans_id
+   and acc_trans.chart_id = chart.id
+   and chart.link like '%paid%'
+   and acc_trans.fx_transaction = '0'
+   and acc_trans.id is null;
+
+update :slschema.acc_trans
+   set id = nextval('paymentid')
+       from :slschema.ap, :slschema.chart
+ where ap.id = acc_trans.trans_id
+   and acc_trans.chart_id = chart.id
+   and chart.link like '%paid%'
+   and acc_trans.fx_transaction = '0'
+   and acc_trans.id is null;
+
+drop sequence :slschema.paymentid;
+
+
+ALTER TABLE :slschema.gl ADD COLUMN curr char(3);
+ALTER TABLE :slschema.gl ADD COLUMN exchangerate float;
+ALTER TABLE :slschema.ap ADD COLUMN exchangerate float;
+ALTER TABLE :slschema.ar ADD COLUMN exchangerate float;
+ALTER TABLE :slschema.oe ADD COLUMN exchangerate float;
+
+UPDATE :slschema.gl
+   SET exchangerate = 1,
+       curr = (select substr(fldvalue, 1, 3)
+                 from defaults
+                where fldname = 'currencies');
+
+UPDATE :slschema.ap
+   SET exchangerate = coalesce((SELECT sell
+                                  FROM :slschema.exchangerate er
+                                 WHERE ap.transdate = er.transdate
+                                   AND ap.curr = er.curr), 1);
+UPDATE :slschema.ar
+   SET exchangerate = coalesce((SELECT buy
+                                  FROM :slschema.exchangerate er
+                                 WHERE ar.transdate = er.transdate
+                                   AND ar.curr = er.curr), 1);
+
+UPDATE :slschema.oe
+   SET exchangerate = coalesce(
+     (SELECT CASE
+             WHEN customer_id > 0 THEN buy
+             WHEN vendor_id > 0 THEN sell
+             END
+        FROM :slschema.exchangerate er
+       WHERE oe.transdate = er.transdate
+             oe.curr = er.curr);
+
+[% END %]
+
 --Accounts
 
 INSERT INTO gifi
@@ -461,12 +543,35 @@ UPDATE :slschema.vendor SET entity_id = (SELECT id FROM entity WHERE 'V-' || ven
 
 UPDATE :slschema.customer SET entity_id = coalesce((SELECT min(id) FROM entity WHERE 'C-' || customernumber = control_code), entity_id);
 
+INSERT INTO defaults(setting_key,value)
+    SELECT 'curr',curr
+    FROM :slschema.curr
+    WHERE rn=1;
+
+INSERT INTO currency(curr,description)
+    SELECT curr,curr
+    FROM  :slschema.curr;
+
+-- Make sure currency table is complete
+INSERT INTO currency(curr,description)
+    SELECT DISTINCT curr, curr
+    FROM (
+              SELECT DISTINCT curr FROM :slschema.ar
+        UNION SELECT DISTINCT curr FROM :slschema.ap
+        UNION SELECT DISTINCT curr FROM :slschema.gl
+    ) xx
+    WHERE curr IS NOT null
+      AND NOT EXISTS (
+        SELECT 1 FROM :slschema.curr c
+        WHERE c.curr = xx.curr
+    );
+
 --Entity Credit Account
 
 UPDATE :slschema.vendor SET business_id = NULL WHERE business_id = 0;
 INSERT INTO entity_credit_account
 (entity_id, meta_number, business_id, creditlimit, ar_ap_account_id,
-        cash_account_id, startdate, enddate, threshold, entity_class)
+        cash_account_id, startdate, enddate, threshold, entity_class, curr)
 SELECT entity_id, vendornumber, business_id, creditlimit,
        (select id
           from account
@@ -476,7 +581,7 @@ SELECT entity_id, vendornumber, business_id, creditlimit,
            from account
            where accno = (select accno from :slschema.chart
                            where id = payment_accno_id)),
-         startdate, enddate, threshold, 1
+         startdate, enddate, threshold, 1, curr
 FROM :slschema.vendor WHERE entity_id IS NOT NULL;
 
 UPDATE :slschema.vendor SET credit_id =
@@ -487,7 +592,7 @@ UPDATE :slschema.vendor SET credit_id =
 UPDATE :slschema.customer SET business_id = NULL WHERE business_id = 0;
 INSERT INTO entity_credit_account
 (entity_id, meta_number, business_id, creditlimit, ar_ap_account_id,
-        cash_account_id, startdate, enddate, threshold, entity_class)
+        cash_account_id, startdate, enddate, threshold, entity_class, curr)
 SELECT entity_id, customernumber, business_id, creditlimit,
        (select id
           from account
@@ -497,7 +602,7 @@ SELECT entity_id, customernumber, business_id, creditlimit,
            from account
            where accno = (select accno from :slschema.chart
                            where id = payment_accno_id)),
-        startdate, enddate, threshold, 2
+        startdate, enddate, threshold, 2, curr
 FROM :slschema.customer WHERE entity_id IS NOT NULL;
 
 UPDATE :slschema.customer SET credit_id =
@@ -588,7 +693,7 @@ group by v.credit_id, v.fax;
 
 -- addresses
 
-INSERT INTO public.country (id, name, short_name) VALUES (-1, 'Invalid Country', 'XX');
+INSERT INTO country (id, name, short_name) VALUES (-1, 'Invalid Country', 'XX');
 
 INSERT INTO eca_to_location(credit_id, location_class, location_id)
 SELECT eca.id, 1,
@@ -729,11 +834,24 @@ WHERE entity_class = 10 AND control_code = 'R-1';
 
 INSERT INTO entity_employee(entity_id, startdate, enddate, role, ssn, sales,
             employeenumber, dob, manager_id)
-    SELECT entity_id, startdate, enddate, r.description, ssn, sales,
-       employeenumber, dob,
-       (SELECT entity_id FROM :slschema.employee WHERE id = em.acsrole_id)
+SELECT entity_id, startdate, enddate,
+       [% IF sl_version < "3.0.0" %]
+         "role",
+       [% ELSE %]
+         r.description,
+       [% END %]
+       ssn, sales,
+           employeenumber, dob,
+           [% IF sl_version < "3.0.0" %]
+                (SELECT ee.entity_id FROM :slschema.employee ee WHERE em.managerid = ee.id)
+           [% ELSE %]
+                (SELECT entity_id FROM :slschema.employee WHERE id = em.acsrole_id)
+           [% END %]
     FROM :slschema.employee em
-    LEFT JOIN :slschema.acsrole r ON em.acsrole_id = r.id;
+[% IF sl_version >= "3.0.0" %]
+     LEFT JOIN :slschema.acsrole r ON em.acsrole_id = r.id
+[% END %]
+;
 
 -- must rebuild this table due to changes since 1.2
 
@@ -747,14 +865,14 @@ drawing, microfiche, partsgroup_id, avgcost)
  SELECT id, partnumber, description, unit,
 listprice, sellprice, lastcost, priceupdate, weight, onhand, notes,
 makemodel, assembly, alternate, rop, (select id
-          from public.account
+          from account
          where accno = (select accno from :slschema.chart
                          where id = inventory_accno_id)),
 (select id
-          from public.account
+          from account
          where accno = (select accno from :slschema.chart
                          where id = income_accno_id)), (select id
-          from public.account
+          from account
          where accno = (select accno from :slschema.chart
                          where id = expense_accno_id)),
  bin, obsolete, bom, image,
@@ -834,15 +952,6 @@ SELECT pg_temp.f_insert_count('sqnumber');
 SELECT pg_temp.f_insert_count('vendornumber');
 SELECT pg_temp.f_insert_count('vinumber');
 
---TODO: Do we need that for MC?
-INSERT INTO defaults(setting_key,value)
-    SELECT 'curr',array_to_string(array_agg(curr),':')
-    FROM :slschema.curr;
-
-INSERT INTO currency(curr,description)
-    SELECT curr,curr
-    FROM  :slschema.curr;
-
 CREATE OR REPLACE FUNCTION pg_temp.f_insert_account(skey varchar(20)) RETURNS VOID AS
 $$
 BEGIN
@@ -874,8 +983,6 @@ SELECT pg_temp.f_insert_account('fxloss_accno_id');
 INSERT INTO assembly (id, parts_id, qty, bom, adj)
 SELECT id, parts_id, qty, bom, adj  FROM :slschema.assembly;
 
-ALTER TABLE gl DISABLE TRIGGER gl_audit_trail;
-
 INSERT INTO business_unit (id, class_id, control_code, description)
 SELECT id, 1, id, description
   FROM :slschema.department;
@@ -896,55 +1003,63 @@ UPDATE business_unit_class
  WHERE id = 2
    AND EXISTS (select 1 from :slschema.project);
 
+-- Prefill approved status of transactions with gl entries
+INSERT INTO transactions (id, table_name, approved)
+SELECT id,'gl',approved
+FROM :slschema.gl;
+
 INSERT INTO gl(id, reference, description, transdate, person_id, notes)
     SELECT gl.id, reference, description, transdate, p.id, gl.notes
       FROM :slschema.gl
  LEFT JOIN :slschema.employee em ON gl.employee_id = em.id
  LEFT JOIN person p ON em.entity_id = p.id;
 
-ALTER TABLE gl ENABLE TRIGGER gl_audit_trail;
-
-ALTER TABLE ar DISABLE TRIGGER ar_audit_trail;
+-- Prefill approved status of transactions with ar entries
+INSERT INTO transactions (id, table_name, approved)
+SELECT ar.id,'ar',approved
+FROM :slschema.ar
+JOIN :slschema.customer ON (ar.customer_id = customer.id) ;
 
 --TODO: Handle amount_tc and netamount_tc
 insert into ar
-(entity_credit_account, person_id,
-        id, invnumber, transdate, taxincluded,
+        (entity_credit_account, person_id,
+        id, invnumber, transdate, crdate, taxincluded,
         amount_bc, netamount_bc,
         amount_tc, netamount_tc,
-[% IF VERSION_COMPARE(lsmbversion,'1.6') < 0; %]
-        paid, datepaid,
-[% END; %]
         duedate, invoice, ordnumber, curr, notes, quonumber, intnotes,
         shipvia, language_code, ponumber, shippingpoint,
         on_hold, approved, reverse, terms, description)
 SELECT
         customer.credit_id,
         (select entity_id from :slschema.employee WHERE id = ar.employee_id),
-        ar.id, invnumber, transdate, ar.taxincluded, amount, netamount,
-        CASE WHEN exchangerate IS NOT NULL THEN amount/exchangerate ELSE 0 END,
-        CASE WHEN exchangerate IS NOT NULL THEN netamount/exchangerate ELSE 0 END,
-[% IF VERSION_COMPARE(lsmbversion,'1.6') < 0; %]
-        paid, datepaid,
-[% END; %]
-        duedate, invoice, ordnumber, ar.curr, ar.notes, quonumber,
-        intnotes,
+        ar.id, invnumber, transdate, transdate, ar.taxincluded, amount, netamount,
+        CASE WHEN exchangerate IS NOT NULL THEN amount/exchangerate ELSE amount END,
+        CASE WHEN exchangerate IS NOT NULL THEN netamount/exchangerate ELSE netamount END,
+        duedate, invoice, ordnumber, ar.curr, ar.notes, quonumber, intnotes,
         shipvia, ar.language_code, ponumber, shippingpoint,
         onhold, approved, case when amount < 0 then true else false end,
         ar.terms, description
-FROM :slschema.ar JOIN :slschema.customer ON (ar.customer_id = customer.id) ;
+[% IF sl_version < "2.8.4" %]
+     FROM (select *, coalesce((SELECT buy
+                                 FROM :slschema.exchangerate er
+                                WHERE ar.transdate = er.transdate
+                                  AND ar.curr = er.curr), 1) as exchangerate
+             FROM :slschema.ar) AS ar
+[% ELSE %]
+     FROM :slschema.ar
+[% END %]
+JOIN :slschema.customer ON (ar.customer_id = customer.id) ;
 
-ALTER TABLE ar ENABLE TRIGGER ar_audit_trail;
-
-ALTER TABLE ap DISABLE TRIGGER ap_audit_trail;
+-- Prefill approved status of transactions with ap entries
+INSERT INTO transactions (id, table_name, approved)
+SELECT ap.id,'ap',approved
+FROM :slschema.ap
+JOIN :slschema.vendor ON (ap.vendor_id = vendor.id) ;
 
 insert into ap
 (entity_credit_account, person_id,
-        id, invnumber, transdate, taxincluded, amount_bc, netamount_bc,
+        id, invnumber, transdate, crdate, taxincluded, amount_bc, netamount_bc,
         amount_tc, netamount_tc,
-[% IF VERSION_COMPARE(lsmbversion,'1.6') < 0; %]
-        paid, datepaid,
-[% END; %]
         duedate, invoice, ordnumber, curr, notes, quonumber, intnotes,
         shipvia, language_code, ponumber, shippingpoint,
         on_hold, approved, reverse, terms, description)
@@ -952,20 +1067,25 @@ SELECT
         vendor.credit_id,
         (select entity_id from :slschema.employee
                 WHERE id = ap.employee_id),
-        ap.id, invnumber, transdate, ap.taxincluded, amount, netamount,
-        CASE WHEN exchangerate IS NOT NULL THEN amount/exchangerate ELSE 0 END,
-        CASE WHEN exchangerate IS NOT NULL THEN netamount/exchangerate ELSE 0 END,
-[% IF VERSION_COMPARE(lsmbversion,'1.6') < 0; %]
-        paid, datepaid,
-[% END; %]
-        duedate, invoice, ordnumber, ap.curr, ap.notes, quonumber,
-        intnotes,
+        ap.id, invnumber, transdate, transdate, ap.taxincluded, amount, netamount,
+        CASE WHEN exchangerate IS NOT NULL THEN amount/exchangerate ELSE amount END,
+        CASE WHEN exchangerate IS NOT NULL THEN netamount/exchangerate ELSE netamount END,
+        duedate, invoice, ordnumber,
+        CASE WHEN exchangerate IS NOT NULL THEN ap.curr ELSE NULL END,
+        ap.notes, quonumber, intnotes,
         shipvia, ap.language_code, ponumber, shippingpoint,
         onhold, approved, case when amount < 0 then true else false end,
         ap.terms, description
-FROM :slschema.ap JOIN :slschema.vendor ON (ap.vendor_id = vendor.id) ;
-
-ALTER TABLE ap ENABLE TRIGGER ap_audit_trail;
+[% IF sl_version < "2.8.4" %]
+     FROM (select *, coalesce((SELECT sell
+                                 FROM :slschema.exchangerate er
+                                WHERE ap.transdate = er.transdate
+                                  AND ap.curr = er.curr), 1) as exchangerate
+             FROM :slschema.ap) AS ap
+[% ELSE %]
+     FROM :slschema.ap
+[% END %]
+    JOIN :slschema.vendor ON (ap.vendor_id = vendor.id) ;
 
 -- ### TODO: there used to be projects here!
 -- ### Move those to business_units
@@ -976,9 +1096,9 @@ INSERT INTO invoice (id, trans_id, parts_id, description, qty, allocated,
     SELECT  id, trans_id, parts_id, description, qty, allocated,
             sellprice, fxsellprice, discount, assemblyitem, unit,
             deliverydate, serialnumber
-       FROM :slschema.invoice;
+      FROM :slschema.invoice
+     WHERE EXISTS (SELECT 1 FROM transactions t WHERE t.id = trans_id);
 
-ALTER TABLE :slschema.acc_trans ADD COLUMN lsmb_entry_id integer;
 
 update :slschema.acc_trans
   set lsmb_entry_id = nextval('acc_trans_entry_id_seq');
@@ -992,22 +1112,26 @@ INSERT INTO acc_trans (entry_id, trans_id, chart_id, amount_bc, amount_tc, curr,
           where accno = (select accno
                            from :slschema.chart
                           where chart.id = ac.chart_id)),
-        CASE WHEN fx_transaction THEN 0 ELSE amount END,
-        CASE WHEN fx_transaction THEN amount ELSE 0 END,
+        ac.amount, ac.amount / coalesce(y.exchangerate, xx.exchangerate, 1),
         xx.curr,
-        transdate, source,
+        ac.transdate, source,
         CASE WHEN cleared IS NOT NULL THEN TRUE ELSE FALSE END,
         memo, approved, cleared, vr_id, invoice.id
    FROM :slschema.acc_trans ac
    JOIN (
-                    SELECT id,curr
-                    FROM (      SELECT id,curr FROM :slschema.ap
-                          UNION SELECT id,curr FROM :slschema.ar
-                          UNION SELECT id,curr FROM :slschema.gl) xx
+                    SELECT id,exchangerate,curr
+                    FROM (      SELECT id,exchangerate,curr FROM :slschema.ap
+                          UNION SELECT id,exchangerate,curr FROM :slschema.ar
+                          UNION SELECT id,exchangerate,curr FROM :slschema.gl) xx
    ) xx ON xx.id=ac.trans_id
    LEFT JOIN :slschema.invoice ON ac.id = invoice.id
                               AND ac.trans_id = invoice.trans_id
-  WHERE chart_id IS NOT NULL
+ LEFT JOIN :slschema.payment y ON (y.trans_id = ac.trans_id AND ac.id = y.id)
+ WHERE (select id
+           from account
+          where accno = (select accno
+                           from :slschema.chart
+                          where chart.id = ac.chart_id)) IS NOT NULL
     AND ac.trans_id IN (SELECT id FROM transactions);
 
 --Payments
@@ -1072,7 +1196,7 @@ FROM :slschema.payment p;
 INSERT INTO cr_coa_to_account(chart_id, account)
 SELECT DISTINCT pc.id, c.description FROM :slschema.acc_trans ac
 JOIN :slschema.chart c ON ac.chart_id = c.id
-JOIN public.account pc on pc.accno = c.accno
+JOIN account pc on pc.accno = c.accno
 WHERE ac.cleared IS NOT NULL
 AND c.link ~ 'paid'
 AND NOT EXISTS (SELECT 1 FROM cr_coa_to_account WHERE chart_id=pc.id);
@@ -1129,7 +1253,7 @@ INSERT INTO cr_report(chart_id, their_total, submitted, end_date, updated, enter
 -- The ID and matching post_date are entered in a temp table to pull the back into cr_report_line immediately after.
 -- Temp table will be dropped automatically at the end of the transaction.
 WITH cr_entry AS (
-SELECT cr.id::INT, cr.end_date, a.source, a.type, a.cleared::TIMESTAMP, a.amount::NUMERIC, a.transdate AS post_date, a.lsmb_entry_id
+SELECT cr.id::INT, cr.end_date, a.source, a.type, a.cleared::DATE, a.amount::NUMERIC, a.transdate AS post_date, a.lsmb_entry_id
     FROM cr_coa_to_account cta
     JOIN account c on cta.chart_id = c.id
     JOIN cr_report cr ON cr.chart_id = c.id
@@ -1164,15 +1288,14 @@ FROM (
 
 -- Patch for suspect clear dates
 -- The UI should reflect this
--- Unsubmit the suspect report to allow easy edition
-UPDATE cr_report SET submitted = false
-WHERE id IN (
-    SELECT DISTINCT report_id FROM cr_report_line
-    WHERE clear_time - post_date > 150
-);
 -- Approve valid reports.
-UPDATE cr_report SET approved = true
-WHERE submitted;
+UPDATE cr_report
+SET approved = true
+WHERE submitted
+  AND id NOT IN (
+    SELECT DISTINCT report_id FROM cr_report_line
+     WHERE NOT (clear_time - post_date BETWEEN 0 AND 150)
+);
 
 -- Log out the Migrator
 DELETE FROM users
@@ -1397,8 +1520,6 @@ SELECT setval('warehouse_id_seq', max(id)) FROM warehouse;
 SELECT setval('partsgroup_id_seq', max(id)) FROM partsgroup;
 SELECT setval('jcitems_id_seq', max(id)) FROM jcitems;
 SELECT setval('payment_type_id_seq', max(id)) FROM payment_type;
-SELECT setval('custom_table_catalog_table_id_seq', max(table_id)) FROM custom_table_catalog;
-SELECT setval('custom_field_catalog_field_id_seq', max(field_id)) FROM custom_field_catalog;
 SELECT setval('menu_node_id_seq', max(id)) FROM menu_node;
 SELECT setval('menu_attribute_id_seq', max(id)) FROM menu_attribute;
 SELECT setval('menu_acl_id_seq', max(id)) FROM menu_acl;
@@ -1425,4 +1546,4 @@ UPDATE defaults SET value = 'yes' where setting_key = 'migration_ok';
 
 COMMIT;
 --TODO:  Translation migration.  Partsgroups?
--- TODO:  User/password Migration
+--TODO:  User/password Migration
